@@ -46,11 +46,14 @@ import java.lang.Math;
  * Code to support the internal GPS receiver via #LocationManager.
  */
 public class InternalGPS
-  implements LocationListener, SensorEventListener, Runnable {
+  implements LocationListener, SensorEventListener, Runnable, AndroidSensor
+{
   private static final String TAG = "XCSoar";
 
   private static Handler handler;
 
+  private final SensorListener listener;
+
   /**
    * Global initialization of the class.  Must be called from the main
    * event thread, because the Handler object must be bound to that
@@ -60,9 +63,6 @@ public class InternalGPS
     handler = new Handler();
   }
 
-  /** the index of this device in the global list */
-  private final int index;
-
   /** the name of the currently selected location provider */
   String locationProvider = LocationManager.GPS_PROVIDER;
   //String locationProvider = LocationManager.NETWORK_PROVIDER;
@@ -74,10 +74,12 @@ public class InternalGPS
   private double acceleration;
   private static boolean queriedLocationSettings = false;
 
+  private int state = STATE_LIMBO;
+
   private final SafeDestruct safeDestruct = new SafeDestruct();
 
-  InternalGPS(Context context, int _index) {
-    index = _index;
+  InternalGPS(Context context, SensorListener listener) {
+    this.listener = listener;
 
     locationManager = (LocationManager)context.getSystemService(Context.LOCATION_SERVICE);
     if (locationManager == null ||
@@ -151,32 +153,29 @@ public class InternalGPS
     update();
   }
 
+  @Override
   public void close() {
     safeDestruct.beginShutdown();
     setLocationProvider(null);
     safeDestruct.finishShutdown();
   }
 
-  private native void setConnected(int connected);
-  private native void setLocation(long time, int n_satellites,
-                                  double longitude, double latitude,
-                                  boolean hasAltitude, double altitude,
-                                  boolean hasBearing, double bearing,
-                                  boolean hasSpeed, double speed,
-                                  boolean hasAccuracy, double accuracy,
-                                  boolean hasAcceleration, double acceleration);
+  @Override
+  public int getState() {
+    return state;
+  }
 
   private void sendLocation(Location location) {
     Bundle extras = location.getExtras();
 
-    setLocation(location.getTime(),
-                extras != null ? extras.getInt("satellites", -1) : -1,
-                location.getLongitude(), location.getLatitude(),
-                location.hasAltitude(), location.getAltitude(),
-                location.hasBearing(), location.getBearing(),
-                location.hasSpeed(), location.getSpeed(),
-                location.hasAccuracy(), location.getAccuracy(),
-                true, acceleration);
+    listener.onLocationSensor(location.getTime(),
+                              extras != null ? extras.getInt("satellites", -1) : -1,
+                              location.getLongitude(), location.getLatitude(),
+                              location.hasAltitude(), location.getAltitude(),
+                              location.hasBearing(), location.getBearing(),
+                              location.hasSpeed(), location.getSpeed(),
+                              location.hasAccuracy(), location.getAccuracy(),
+                              true, acceleration);
   }
 
   private void setConnectedSafe(int connected) {
@@ -184,7 +183,7 @@ public class InternalGPS
       return;
 
     try {
-      setConnected(connected);
+      listener.onConnected(connected);
     } finally {
       safeDestruct.Decrement();
     }
@@ -196,7 +195,7 @@ public class InternalGPS
       return;
 
     try {
-      setConnected(2); // fix found
+      listener.onConnected(2); // fix found
       sendLocation(newLocation);
     } finally {
       safeDestruct.Decrement();
@@ -218,14 +217,20 @@ public class InternalGPS
                                         Bundle extras) {
     switch (status) {
     case LocationProvider.OUT_OF_SERVICE:
+      state = STATE_FAILED;
       setConnectedSafe(0); // not connected
+      listener.onSensorStateChanged();
       break;
 
     case LocationProvider.TEMPORARILY_UNAVAILABLE:
+      state = STATE_LIMBO;
       setConnectedSafe(1); // waiting for fix
+      listener.onSensorStateChanged();
       break;
 
     case LocationProvider.AVAILABLE:
+      state = STATE_READY;
+      listener.onSensorStateChanged();
       break;
     }
   }
