@@ -55,7 +55,7 @@ public class XCSoar extends Activity {
    * Hack: this is set by onCreate(), to support the "testing"
    * package.
    */
-  protected static Class serviceClass;
+  protected static Class<?> serviceClass;
 
   private static NativeView nativeView;
 
@@ -96,9 +96,6 @@ public class XCSoar extends Activity {
 
     IOIOHelper.onCreateContext(this);
 
-    BluetoothHelper.Initialize(this);
-    UsbSerialHelper.Initialise(this);
-
     // fullscreen mode
     requestWindowFeature(Window.FEATURE_NO_TITLE);
     getWindow().addFlags(WindowManager.LayoutParams.FLAG_FULLSCREEN|
@@ -287,8 +284,6 @@ public class XCSoar extends Activity {
 
     Log.d(TAG, "in onDestroy()");
 
-    UsbSerialHelper.Deinitialise(this);
-
     if (batteryReceiver != null) {
       unregisterReceiver(batteryReceiver);
       batteryReceiver = null;
