@@ -24,38 +24,24 @@ Copyright_License {
 #ifndef XCSOAR_ANDROID_BMP085_DEVICE_HPP
 #define XCSOAR_ANDROID_BMP085_DEVICE_HPP
 
-#include "BMP085Listener.hpp"
 #include "java/Object.hxx"
-#include "Math/SelfTimingKalmanFilter1d.hpp"
-#include "util/Compiler.h"
 
-#include <jni.h>
+class SensorListener;
 
-class BMP085Device final : private BMP085Listener {
-  unsigned index;
-  Java::GlobalObject obj;
+namespace BMP085Device {
 
-  /**
-   * This Kalman filter is used to smooth the pressure input.
-   */
-  SelfTimingKalmanFilter1d kalman_filter;
+void
+Initialise(JNIEnv *env) noexcept;
 
-public:
-  static void Initialise(JNIEnv *env);
-  static void Deinitialise(JNIEnv *env);
+void
+Deinitialise(JNIEnv *env) noexcept;
 
-  BMP085Device(unsigned index,
-               JNIEnv *env, jobject holder,
-               unsigned twi_num, unsigned eoc_pin,
-               unsigned oversampling);
+Java::LocalObject
+Create(JNIEnv *env, jobject holder,
+       unsigned twi_num, unsigned eoc_pin,
+       unsigned oversampling,
+       SensorListener &listener);
 
-  ~BMP085Device();
-
-private:
-  /* virtual methods from class BMP085Listener */
-  virtual void onBMP085Values(double temperature,
-                              AtmosphericPressure pressure) override;
-  virtual void onBMP085Error() override;
-};
+} // BMP085Device
 
 #endif
