@@ -30,12 +30,14 @@ Copyright_License {
 #include "Vibrator.hpp"
 #include "InternalSensors.hpp"
 #include "GliderLink.hpp"
+#include "Sensor.hpp"
 #include "PortBridge.hpp"
 #include "BluetoothHelper.hpp"
 #include "UsbSerialHelper.hpp"
-#include "NativeLeScanCallback.hpp"
+#include "NativeDetectDeviceListener.hpp"
 #include "NativePortListener.hpp"
 #include "NativeInputListener.hpp"
+#include "NativeSensorListener.hpp"
 #include "TextUtil.hpp"
 #include "Product.hpp"
 #include "Nook.hpp"
@@ -59,6 +61,7 @@ Copyright_License {
 #include "java/File.hxx"
 #include "java/InputStream.hxx"
 #include "java/URL.hxx"
+#include "java/Closeable.hxx"
 #include "util/Compiler.h"
 #include "org_xcsoar_NativeView.h"
 #include "io/async/GlobalAsioThread.hpp"
@@ -68,13 +71,9 @@ Copyright_License {
 #include "util/Exception.hxx"
 
 #include "IOIOHelper.hpp"
-#include "NativeBMP085Listener.hpp"
 #include "BMP085Device.hpp"
-#include "NativeI2CbaroListener.hpp"
 #include "I2CbaroDevice.hpp"
-#include "NativeNunchuckListener.hpp"
 #include "NunchuckDevice.hpp"
-#include "NativeVoltageListener.hpp"
 #include "VoltageDevice.hpp"
 
 #ifndef NDEBUG
@@ -96,6 +95,8 @@ NativeView *native_view;
 Vibrator *vibrator;
 bool os_haptic_feedback_enabled;
 
+BluetoothHelper *bluetooth_helper;
+UsbSerialHelper *usb_serial_helper;
 IOIOHelper *ioio_helper;
 
 gcc_visibility_default
@@ -118,28 +119,27 @@ try {
   Java::Object::Initialise(env);
   Java::File::Initialise(env);
   Java::InputStream::Initialise(env);
+  Java::InitialiseCloseable(env);
   Java::URL::Initialise(env);
   Java::URLConnection::Initialise(env);
 
   NativeView::Initialise(env);
   Environment::Initialise(env);
   AndroidBitmap::Initialise(env);
+  NativeSensorListener::Initialise(env);
   InternalSensors::Initialise(env);
   GliderLink::Initialise(env);
   NativePortListener::Initialise(env);
   NativeInputListener::Initialise(env);
+  AndroidSensor::Initialise(env);
   PortBridge::Initialise(env);
-  BluetoothHelper::Initialise(env);
-  UsbSerialHelper::Initialise(env);
-  NativeLeScanCallback::Initialise(env);
+  const bool have_bluetooth = BluetoothHelper::Initialise(env);
+  const bool have_usb_serial = UsbSerialHelper::Initialise(env);
+  NativeDetectDeviceListener::Initialise(env);
   const bool have_ioio = IOIOHelper::Initialise(env);
-  NativeBMP085Listener::Initialise(env);
   BMP085Device::Initialise(env);
-  NativeI2CbaroListener::Initialise(env);
   I2CbaroDevice::Initialise(env);
-  NativeNunchuckListener::Initialise(env);
   NunchuckDevice::Initialise(env);
-  NativeVoltageListener::Initialise(env);
   VoltageDevice::Initialise(env);
 
   context = new Context(env, _context);
@@ -164,6 +164,22 @@ try {
   Vibrator::Initialise(env);
   vibrator = Vibrator::Create(env, *context);
 
+  if (have_bluetooth) {
+    try {
+      bluetooth_helper = new BluetoothHelper(env, *context);
+    } catch (...) {
+      LogError(std::current_exception(), "Failed to initialise Bluetooth");
+    }
+  }
+
+  if (have_usb_serial) {
+    try {
+      usb_serial_helper = new UsbSerialHelper(env, *context);
+    } catch (...) {
+      LogError(std::current_exception(), "Failed to initialise USB serial support");
+    }
+  }
+
   if (have_ioio) {
     try {
       ioio_helper = new IOIOHelper(env);
@@ -229,6 +245,12 @@ Java_org_xcsoar_NativeView_deinitializeNative(JNIEnv *env, jobject obj)
   delete ioio_helper;
   ioio_helper = nullptr;
 
+  delete usb_serial_helper;
+  usb_serial_helper = nullptr;
+
+  delete bluetooth_helper;
+  bluetooth_helper = nullptr;
+
   delete vibrator;
   vibrator = nullptr;
 
@@ -247,20 +269,17 @@ Java_org_xcsoar_NativeView_deinitializeNative(JNIEnv *env, jobject obj)
   context = nullptr;
 
   BMP085Device::Deinitialise(env);
-  NativeBMP085Listener::Deinitialise(env);
   I2CbaroDevice::Deinitialise(env);
-  NativeI2CbaroListener::Deinitialise(env);
   NunchuckDevice::Deinitialise(env);
-  NativeNunchuckListener::Deinitialise(env);
   VoltageDevice::Deinitialise(env);
-  NativeVoltageListener::Deinitialise(env);
   IOIOHelper::Deinitialise(env);
-  NativeLeScanCallback::Deinitialise(env);
+  NativeDetectDeviceListener::Deinitialise(env);
   UsbSerialHelper::Deinitialise(env);
   BluetoothHelper::Deinitialise(env);
   NativeInputListener::Deinitialise(env);
   NativePortListener::Deinitialise(env);
   InternalSensors::Deinitialise(env);
+  NativeSensorListener::Deinitialise(env);
   GliderLink::Deinitialise(env);
   AndroidBitmap::Deinitialise(env);
   Environment::Deinitialise(env);
