@@ -2,7 +2,7 @@
 Copyright_License {
 
   XCSoar Glide Computer - http://www.xcsoar.org/
-  Copyright (C) 2000-2015 The XCSoar Project
+  Copyright (C) 2000-2021 The XCSoar Project
   A detailed list of copyright holders can be found in the file "AUTHORS".
 
   This program is free software; you can redistribute it and/or
@@ -27,10 +27,13 @@ Copyright_License {
 #include <jni.h>
 
 namespace SoundUtil {
-  void Initialise(JNIEnv *env);
-  void Deinitialise(JNIEnv *env);
 
-  bool Play(JNIEnv *env, jobject context, const char *name);
-};
+void Initialise(JNIEnv *env);
+void Deinitialise(JNIEnv *env);
+
+bool Play(JNIEnv *env, jobject context, const char *name);
+bool PlayExternal(JNIEnv *env, jobject context, const char *path);
+
+} // namespace SoundUtil
 
 #endif
