@@ -2,7 +2,7 @@
 Copyright_License {
 
   XCSoar Glide Computer - http://www.xcsoar.org/
-  Copyright (C) 2000-2015 The XCSoar Project
+  Copyright (C) 2000-2021 The XCSoar Project
   A detailed list of copyright holders can be found in the file "AUTHORS".
 
   This program is free software; you can redistribute it and/or
@@ -24,13 +24,22 @@ Copyright_License {
 #ifndef XCSOAR_DEVICE_CONFIGURED_PORT_HPP
 #define XCSOAR_DEVICE_CONFIGURED_PORT_HPP
 
+#include <memory>
+
+namespace Cares { class Channel; }
+class EventLoop;
 class Port;
 class PortListener;
 class DataHandler;
 struct DeviceConfig;
 
-Port *
-OpenPort(const DeviceConfig &config, PortListener *listener,
+/**
+ * Open the port described by #DeviceConfig.  On error, throws a
+ * #std::runtime_error or returns nullptr.
+ */
+std::unique_ptr<Port>
+OpenPort(EventLoop &event_loop, Cares::Channel &cares,
+         const DeviceConfig &config, PortListener *listener,
          DataHandler &handler);
 
 #endif
