@@ -26,10 +26,12 @@ Copyright_License {
 #include "Form/DataField/GeoPoint.hpp"
 #include "Form/DataField/RoughTime.hpp"
 #include "Form/DataField/Prefix.hpp"
+#include "Form/DataField/Date.hpp"
 #include "ComboPicker.hpp"
 #include "Dialogs/TextEntry.hpp"
 #include "Dialogs/TimeEntry.hpp"
 #include "Dialogs/GeoPointEntry.hpp"
+#include "Dialogs/DateEntry.hpp"
 
 bool
 EditDataFieldDialog(const TCHAR *caption, DataField &df,
@@ -57,6 +59,14 @@ EditDataFieldDialog(const TCHAR *caption, DataField &df,
 
     gdf.ModifyValue(value);
     return true;
+  } else if (df.GetType() == DataField::Type::DATE) {
+    auto &dfd = static_cast<DataFieldDate &>(df);
+    BrokenDate date = dfd.GetValue();
+    if (!DateEntryDialog(caption, date, true))
+      return false;
+
+    dfd.SetValue(date);
+    return true;
   } else {
     const TCHAR *value = df.GetAsString();
     if (value == NULL)
