@@ -2,7 +2,7 @@
 Copyright_License {
 
   XCSoar Glide Computer - http://www.xcsoar.org/
-  Copyright (C) 2000-2015 The XCSoar Project
+  Copyright (C) 2000-2021 The XCSoar Project
   A detailed list of copyright holders can be found in the file "AUTHORS".
 
   This program is free software; you can redistribute it and/or
@@ -27,7 +27,7 @@ Copyright_License {
 #include "Widget/TwoWidgets.hpp"
 #include "Widget/RowFormWidget.hpp"
 #include "Renderer/TwoTextRowsRenderer.hpp"
-#include "Screen/Canvas.hpp"
+#include "ui/canvas/Canvas.hpp"
 #include "Screen/Layout.hpp"
 #include "Form/DataField/Prefix.hpp"
 #include "Form/DataField/Listener.hpp"
@@ -36,7 +36,7 @@ Copyright_License {
 #include "FLARM/FlarmId.hpp"
 #include "FLARM/Global.hpp"
 #include "FLARM/TrafficDatabases.hpp"
-#include "Util/StaticString.hxx"
+#include "util/StaticString.hxx"
 #include "Language/Language.hpp"
 #include "UIGlobals.hpp"
 #include "Look/DialogLook.hpp"
@@ -63,7 +63,7 @@ enum Buttons {
 class TrafficListButtons;
 
 class TrafficListWidget : public ListWidget, public DataFieldListener,
-                          public ActionListener, NullBlackboardListener {
+                          NullBlackboardListener {
   struct Item {
     /**
      * The FLARM traffic id.  If this is "undefined", then this object
@@ -71,11 +71,11 @@ class TrafficListWidget : public ListWidget, public DataFieldListener,
      */
     FlarmId id;
 
-#ifdef HAVE_SKYLINES_TRACKING_HANDLER
+#ifdef HAVE_SKYLINES_TRACKING
     /**
      * The SkyLines account id.
      */
-    uint32_t skylines_id;
+    uint32_t skylines_id = 0;
 
     uint32_t time_of_day_ms;
 #endif
@@ -84,14 +84,14 @@ class TrafficListWidget : public ListWidget, public DataFieldListener,
      * The color that was assigned by the user to this FLARM peer.  It
      * is FlarmColor::COUNT if the color has not yet been determined.
      */
-    FlarmColor color;
+    FlarmColor color = FlarmColor::COUNT;
 
     /**
      * Were the attributes below already lazy-loaded from the
      * database?  We can't use nullptr for this, because both will be
      * nullptr after a failed lookup.
      */
-    bool loaded;
+    bool loaded = false;
 
     const FlarmNetRecord *record;
     const TCHAR *callsign;
@@ -99,47 +99,40 @@ class TrafficListWidget : public ListWidget, public DataFieldListener,
     /**
      * This object's location.  Check GeoPoint::IsValid().
      */
-    GeoPoint location;
+    GeoPoint location = GeoPoint::Invalid();
 
     /**
      * The vector from the current aircraft location to this object's
      * location (if known).  Check GeoVector::IsValid().
      */
-    GeoVector vector;
+    GeoVector vector = GeoVector::Invalid();
 
     /**
      * The display name of the SkyLines account.
      */
-    std::string name;
+    tstring name;
 
-#ifdef HAVE_SKYLINES_TRACKING_HANDLER
+#ifdef HAVE_SKYLINES_TRACKING
     StaticString<20> near_name;
-    fixed near_distance;
+    double near_distance;
 
     int altitude;
 #endif
 
     explicit Item(FlarmId _id)
-      :id(_id),
-#ifdef HAVE_SKYLINES_TRACKING_HANDLER
-       skylines_id(0),
-#endif
-       color(FlarmColor::COUNT),
-       loaded(false),
-       location(GeoPoint::Invalid()),
-       vector(GeoVector::Invalid()) {
+      :id(_id) {
       assert(id.IsDefined());
       assert(IsFlarm());
 
-#ifdef HAVE_SKYLINES_TRACKING_HANDLER
+#ifdef HAVE_SKYLINES_TRACKING
       near_name.clear();
 #endif
     }
 
-#ifdef HAVE_SKYLINES_TRACKING_HANDLER
+#ifdef HAVE_SKYLINES_TRACKING
     explicit Item(uint32_t _id, uint32_t _time_of_day_ms,
                   const GeoPoint &_location, int _altitude,
-                  std::string &&_name)
+                  tstring &&_name)
       :id(FlarmId::Undefined()), skylines_id(_id),
        time_of_day_ms(_time_of_day_ms),
        color(FlarmColor::COUNT),
@@ -160,7 +153,7 @@ class TrafficListWidget : public ListWidget, public DataFieldListener,
       return id.IsDefined();
     }
 
-#ifdef HAVE_SKYLINES_TRACKING_HANDLER
+#ifdef HAVE_SKYLINES_TRACKING
     /**
      * Does this object describe data from SkyLines live tracking?
      */
@@ -173,7 +166,7 @@ class TrafficListWidget : public ListWidget, public DataFieldListener,
       if (IsFlarm()) {
         record = traffic_databases->flarm_net.FindRecordById(id);
         callsign = traffic_databases->FindNameById(id);
-#ifdef HAVE_SKYLINES_TRACKING_HANDLER
+#ifdef HAVE_SKYLINES_TRACKING
       } else if (IsSkyLines()) {
         record = nullptr;
         callsign = nullptr;
@@ -196,7 +189,7 @@ class TrafficListWidget : public ListWidget, public DataFieldListener,
 
   typedef std::vector<Item> ItemList;
 
-  ActionListener &action_listener;
+  WndForm &dialog;
 
   const RowFormWidget *const filter_widget;
 
@@ -213,9 +206,9 @@ class TrafficListWidget : public ListWidget, public DataFieldListener,
   TwoTextRowsRenderer row_renderer;
 
 public:
-  TrafficListWidget(ActionListener &_action_listener,
+  TrafficListWidget(WndForm &_dialog,
                     const FlarmId *array, size_t count)
-    :action_listener(_action_listener), filter_widget(nullptr),
+    :dialog(_dialog), filter_widget(nullptr),
      buttons(nullptr) {
     items.reserve(count);
 
@@ -223,10 +216,10 @@ public:
       items.emplace_back(array[i]);
   }
 
-  TrafficListWidget(ActionListener &_action_listener,
+  TrafficListWidget(WndForm &_dialog,
                     const RowFormWidget &_filter_widget,
                     TrafficListButtons &_buttons)
-    :action_listener(_action_listener), filter_widget(&_filter_widget),
+    :dialog(_dialog), filter_widget(&_filter_widget),
      buttons(&_buttons) {
   }
 
@@ -274,18 +267,24 @@ private:
   void UpdateButtons();
 
   void OpenDetails(unsigned index);
+
   void OpenMap(unsigned index);
 
 public:
-  /* virtual methods from class Widget */
-
-  virtual void Prepare(ContainerWindow &parent,
-                       const PixelRect &rc) override;
-  virtual void Unprepare() override {
-    DeleteWindow();
+  void OpenDetails() noexcept {
+    OpenDetails(GetList().GetCursorIndex());
   }
 
-  virtual void Show(const PixelRect &rc) override {
+  void OpenMap() noexcept {
+    OpenMap(GetList().GetCursorIndex());
+  }
+
+  /* virtual methods from class Widget */
+
+  void Prepare(ContainerWindow &parent,
+               const PixelRect &rc) noexcept override;
+
+  void Show(const PixelRect &rc) noexcept override {
     ListWidget::Show(rc);
 
     if (filter_widget != nullptr)
@@ -294,7 +293,7 @@ public:
     CommonInterface::GetLiveBlackboard().AddListener(*this);
   }
 
-  virtual void Hide() override {
+  void Hide() noexcept override {
     CommonInterface::GetLiveBlackboard().RemoveListener(*this);
 
     ListWidget::Hide();
@@ -302,27 +301,24 @@ public:
 
   /* virtual methods from ListItemRenderer */
   virtual void OnPaintItem(Canvas &canvas, const PixelRect rc,
-                           unsigned idx) override;
+                           unsigned idx) noexcept override;
 
   /* virtual methods from ListCursorHandler */
-  virtual void OnCursorMoved(unsigned index) override {
+  virtual void OnCursorMoved(unsigned index) noexcept override {
     UpdateButtons();
   }
 
-  virtual bool CanActivateItem(unsigned index) const override {
+  virtual bool CanActivateItem(unsigned index) const noexcept override {
     return true;
   }
 
-  virtual void OnActivateItem(unsigned index) override;
+  virtual void OnActivateItem(unsigned index) noexcept override;
 
   /* virtual methods from DataFieldListener */
-  virtual void OnModified(DataField &df) override {
+  void OnModified(DataField &df) noexcept override {
     UpdateList();
   }
 
-  /* virtual methods from ActionListener */
-  virtual void OnAction(int id) override;
-
 private:
   /* virtual methods from BlackboardListener */
   virtual void OnGPSUpdate(const MoreData &basic) override {
@@ -341,30 +337,30 @@ public:
     listener = _listener;
   }
 
-  virtual void Prepare(ContainerWindow &parent,
-                       const PixelRect &rc) override {
+  void Prepare(ContainerWindow &parent,
+               const PixelRect &rc) noexcept override {
     PrefixDataField *callsign_df = new PrefixDataField(_T(""), listener);
     Add(_("Competition ID"), nullptr, callsign_df);
   }
 };
 
 class TrafficListButtons : public RowFormWidget {
-  ActionListener &dialog;
-  ActionListener *list;
+  WndForm &dialog;
+  TrafficListWidget *list;
 
 public:
-  TrafficListButtons(const DialogLook &look, ActionListener &_dialog)
+  TrafficListButtons(const DialogLook &look, WndForm &_dialog)
     :RowFormWidget(look), dialog(_dialog) {}
 
-  void SetList(ActionListener *_list) {
+  void SetList(TrafficListWidget *_list) noexcept {
     list = _list;
   }
 
-  virtual void Prepare(ContainerWindow &parent,
-                       const PixelRect &rc) override {
-    AddButton(_("Details"), *list, DETAILS);
-    AddButton(_("Map"), *list, MAP);
-    AddButton(_("Close"), dialog, mrCancel);
+  void Prepare(ContainerWindow &parent,
+               const PixelRect &rc) noexcept override {
+    AddButton(_("Details"), [this](){ list->OpenDetails(); });
+    AddButton(_("Map"), [this](){ list->OpenMap(); });
+    AddButton(_("Close"), dialog.MakeModalResultCallback(mrCancel));
   }
 };
 
@@ -403,17 +399,17 @@ TrafficListWidget::UpdateList()
       AddItem(i.id);
     }
 
-#ifdef HAVE_SKYLINES_TRACKING_HANDLER
+#ifdef HAVE_SKYLINES_TRACKING
     /* show SkyLines traffic unless this is a FLARM traffic picker
        dialog (from dlgTeamCode) */
     if (buttons != nullptr) {
       const auto &data = tracking->GetSkyLinesData();
-      const ScopeLock protect(data.mutex);
+      const std::lock_guard<Mutex> lock(data.mutex);
       for (const auto &i : data.traffic) {
         const auto name_i = data.user_names.find(i.first);
-        std::string name = name_i != data.user_names.end()
+        tstring name = name_i != data.user_names.end()
           ? name_i->second
-          : std::string();
+          : tstring();
 
         items.emplace_back(i.first, i.second.time_of_day_ms,
                            i.second.location, i.second.altitude,
@@ -425,8 +421,8 @@ TrafficListWidget::UpdateList()
             item.vector = GeoVector(CommonInterface::Basic().location,
                                     i.second.location);
 
-          const auto *wp = way_points.GetNearestLandable(i.second.location,
-                                                         fixed(20000));
+          const auto wp = way_points.GetNearestLandable(i.second.location,
+                                                        20000);
           if (wp != nullptr) {
             item.near_name = wp->name.c_str();
             item.near_distance = wp->location.DistanceS(i.second.location);
@@ -481,10 +477,10 @@ TrafficListWidget::UpdateVolatile()
         i.location.SetInvalid();
         i.vector.SetInvalid();
       }
-#ifdef HAVE_SKYLINES_TRACKING_HANDLER
+#ifdef HAVE_SKYLINES_TRACKING
     } else if (i.IsSkyLines()) {
       const auto &data = tracking->GetSkyLinesData();
-      const ScopeLock protect(data.mutex);
+      const std::lock_guard<Mutex> lock(data.mutex);
 
       auto live = data.traffic.find(i.skylines_id);
       if (live != data.traffic.end()) {
@@ -532,7 +528,7 @@ TrafficListWidget::UpdateButtons()
 
 void
 TrafficListWidget::Prepare(ContainerWindow &parent,
-                           const PixelRect &rc)
+                           const PixelRect &rc) noexcept
 {
   const DialogLook &look = UIGlobals::GetDialogLook();
   ListControl &list = CreateList(parent, look, rc,
@@ -545,14 +541,14 @@ TrafficListWidget::Prepare(ContainerWindow &parent,
     list.SetLength(items.size());
 }
 
-#ifdef HAVE_SKYLINES_TRACKING_HANDLER
+#ifdef HAVE_SKYLINES_TRACKING
 
 /**
  * Calculate how many minutes have passed since #past_ms.
  */
 gcc_const
 static unsigned
-SinceInMinutes(fixed now_s, uint32_t past_ms)
+SinceInMinutes(double now_s, uint32_t past_ms)
 {
   const unsigned day_minutes = 24 * 60;
   unsigned now_minutes = uint32_t(now_s / 60) % day_minutes;
@@ -570,25 +566,15 @@ SinceInMinutes(fixed now_s, uint32_t past_ms)
 
 #endif
 
-/**
- * Draw right-aligned text.
- */
-static void
-DrawTextRight(Canvas &canvas, int x, int y, const TCHAR *text)
-{
-  unsigned width = canvas.CalcTextWidth(text);
-  canvas.DrawText(x - width, y, text);
-}
-
 void
-TrafficListWidget::OnPaintItem(Canvas &canvas, const PixelRect rc,
-                               unsigned index)
+TrafficListWidget::OnPaintItem(Canvas &canvas, PixelRect rc,
+                               unsigned index) noexcept
 {
   assert(index < items.size());
   Item &item = items[index];
 
   assert(item.IsFlarm()
-#ifdef HAVE_SKYLINES_TRACKING_HANDLER
+#ifdef HAVE_SKYLINES_TRACKING
          || item.IsSkyLines()
 #endif
          );
@@ -620,7 +606,7 @@ TrafficListWidget::OnPaintItem(Canvas &canvas, const PixelRect rc,
       tmp.Format(_T("%s - %s"), callsign, tmp_id);
     else
       tmp.Format(_T("%s"), tmp_id);
-#ifdef HAVE_SKYLINES_TRACKING_HANDLER
+#ifdef HAVE_SKYLINES_TRACKING
   } else if (item.IsSkyLines()) {
     if (!item.name.empty())
       tmp = item.name.c_str();
@@ -656,16 +642,23 @@ TrafficListWidget::OnPaintItem(Canvas &canvas, const PixelRect rc,
     canvas.SelectHollowBrush();
 
     const PixelSize size = canvas.CalcTextSize(tmp);
-    canvas.Rectangle(rc.left + row_renderer.GetX() - frame_padding,
-                     rc.top + row_renderer.GetFirstY() - frame_padding,
-                     rc.left + row_renderer.GetX() + size.cx + frame_padding,
-                     rc.top + row_renderer.GetFirstY() + size.cy + frame_padding);
+    canvas.DrawRectangle(PixelRect{{rc.left + row_renderer.GetX(), rc.top + row_renderer.GetFirstY()}, size}.WithMargin(frame_padding));
   }
 
   row_renderer.DrawFirstRow(canvas, rc, tmp);
 
   canvas.Select(small_font);
 
+  /* draw bearing and distance on the right */
+  if (item.vector.IsValid()) {
+    row_renderer.DrawRightFirstRow(canvas, rc,
+                                            FormatUserDistanceSmart(item.vector.distance).c_str());
+
+    // Draw leg bearing
+    rc.right = row_renderer.DrawRightSecondRow(canvas, rc,
+                                               FormatBearing(item.vector.bearing).c_str());
+  }
+
   if (record != nullptr) {
     tmp.clear();
 
@@ -688,7 +681,7 @@ TrafficListWidget::OnPaintItem(Canvas &canvas, const PixelRect rc,
 
     if (!tmp.empty())
       row_renderer.DrawSecondRow(canvas, rc, tmp);
-#ifdef HAVE_SKYLINES_TRACKING_HANDLER
+#ifdef HAVE_SKYLINES_TRACKING
   } else if (item.IsSkyLines()) {
     if (CommonInterface::Basic().time_available) {
       tmp.UnsafeFormat(_("%u minutes ago"),
@@ -704,24 +697,12 @@ TrafficListWidget::OnPaintItem(Canvas &canvas, const PixelRect rc,
 
     if (!tmp.empty())
       tmp.append(_T("; "));
-    tmp.append(FormatUserAltitude(fixed(item.altitude)));
+    tmp.append(FormatUserAltitude(item.altitude));
 
     if (!tmp.empty())
       row_renderer.DrawSecondRow(canvas, rc, tmp);
 #endif
   }
-
-  /* draw bearing and distance on the right */
-  if (item.vector.IsValid()) {
-    DrawTextRight(canvas, rc.right - text_padding,
-                  rc.top + row_renderer.GetFirstY(),
-                  FormatUserDistanceSmart(item.vector.distance).c_str());
-
-    // Draw leg bearing
-    DrawTextRight(canvas, rc.right - text_padding,
-                  rc.top + row_renderer.GetSecondY(),
-                  FormatBearing(item.vector.bearing).c_str());
-  }
 }
 
 void
@@ -749,55 +730,48 @@ TrafficListWidget::OpenMap(unsigned index)
     return;
 
   if (PanTo(item.location))
-    action_listener.OnAction(mrCancel);
+    dialog.SetModalResult(mrCancel);
 }
 
 void
-TrafficListWidget::OnActivateItem(unsigned index)
+TrafficListWidget::OnActivateItem(unsigned index) noexcept
 {
   if (buttons == nullptr)
     /* it's a traffic picker: finish the dialog */
-    action_listener.OnAction(mrOK);
+    dialog.SetModalResult(mrOK);
   else
     OpenDetails(index);
 }
 
-void
-TrafficListWidget::OnAction(int id)
-{
-  switch (Buttons(id)) {
-  case DETAILS:
-    OpenDetails(GetList().GetCursorIndex());
-    break;
-
-  case MAP:
-    OpenMap(GetList().GetCursorIndex());
-    break;
-  }
-}
-
 void
 TrafficListDialog()
 {
   const DialogLook &look = UIGlobals::GetDialogLook();
-  WidgetDialog dialog(look);
 
-  TrafficFilterWidget *filter_widget = new TrafficFilterWidget(look);
+  WidgetDialog dialog(WidgetDialog::Full{}, UIGlobals::GetMainWindow(),
+                      look, _("Traffic"));
 
-  TrafficListButtons *buttons_widget = new TrafficListButtons(look, dialog);
+  auto filter_widget = std::make_unique<TrafficFilterWidget>(look);
 
-  TwoWidgets *left_widget =
-    new TwoWidgets(filter_widget, buttons_widget, true);
+  auto buttons_widget = std::make_unique<TrafficListButtons>(look, dialog);
 
-  TrafficListWidget *const list_widget =
-    new TrafficListWidget(dialog, *filter_widget, *buttons_widget);
+  auto list_widget =
+    std::make_unique<TrafficListWidget>(dialog, *filter_widget,
+                                        *buttons_widget);
 
-  filter_widget->SetListener(list_widget);
-  buttons_widget->SetList(list_widget);
+  filter_widget->SetListener(list_widget.get());
+  buttons_widget->SetList(list_widget.get());
 
-  TwoWidgets *widget = new TwoWidgets(left_widget, list_widget, false);
+  auto left_widget =
+    std::make_unique<TwoWidgets>(std::move(filter_widget),
+                                 std::move(buttons_widget),
+                                 true);
 
-  dialog.CreateFull(UIGlobals::GetMainWindow(), _("Traffic"), widget);
+  auto widget = std::make_unique<TwoWidgets>(std::move(left_widget),
+                                             std::move(list_widget),
+                                             false);
+
+  dialog.FinishPreliminary(widget.release());
   dialog.ShowModal();
 }
 
@@ -806,17 +780,18 @@ PickFlarmTraffic(const TCHAR *title, FlarmId array[], unsigned count)
 {
   assert(count > 0);
 
-  WidgetDialog dialog(UIGlobals::GetDialogLook());
+  WidgetDialog dialog(WidgetDialog::Full{}, UIGlobals::GetMainWindow(),
+                      UIGlobals::GetDialogLook(), title);
 
   TrafficListWidget *const list_widget =
     new TrafficListWidget(dialog, array, count);
 
   Widget *widget = list_widget;
 
-  dialog.CreateFull(UIGlobals::GetMainWindow(), title, widget);
   dialog.AddButton(_("Select"), mrOK);
   dialog.AddButton(_("Cancel"), mrCancel);
   dialog.EnableCursorSelection();
+  dialog.FinishPreliminary(widget);
 
   return dialog.ShowModal() == mrOK
     ? list_widget->GetCursorId()
