@@ -2,7 +2,7 @@
 Copyright_License {
 
   XCSoar Glide Computer - http://www.xcsoar.org/
-  Copyright (C) 2000-2015 The XCSoar Project
+  Copyright (C) 2000-2021 The XCSoar Project
   A detailed list of copyright holders can be found in the file "AUTHORS".
 
   This program is free software; you can redistribute it and/or
@@ -31,8 +31,8 @@ Copyright_License {
 #include "NMEA/Derived.hpp"
 #include "Computer/Settings.hpp"
 #include "Math/SunEphemeris.hpp"
-#include "Util/StaticString.hxx"
-#include "Util/Macros.hpp"
+#include "util/StaticString.hxx"
+#include "util/Macros.hpp"
 #include "Language/Language.hpp"
 #include "Interface.hpp"
 #include "Formatter/UserUnits.hpp"
@@ -41,7 +41,8 @@ Copyright_License {
 
 static const TCHAR *
 FormatGlideResult(TCHAR *buffer, size_t size,
-                  const GlideResult &result, const GlideSettings &settings)
+                  const GlideResult &result,
+                  const GlideSettings &settings) noexcept
 {
   switch (result.validity) {
   case GlideResult::Validity::OK:
@@ -62,7 +63,7 @@ FormatGlideResult(TCHAR *buffer, size_t size,
 
 void
 WaypointInfoWidget::AddGlideResult(const TCHAR *label,
-                                   const GlideResult &result)
+                                   const GlideResult &result) noexcept
 {
   const ComputerSettings &settings = CommonInterface::GetComputerSettings();
 
@@ -74,13 +75,13 @@ WaypointInfoWidget::AddGlideResult(const TCHAR *label,
 
 gcc_const
 static BrokenTime
-BreakHourOfDay(fixed t)
+BreakHourOfDay(double t) noexcept
 {
   /* depending on the time zone, the SunEphemeris library may return a
      negative time of day; the following check catches this before we
      cast the value to "unsigned" */
-  if (negative(t))
-    t += fixed(24);
+  if (t < 0)
+    t += 24;
 
   unsigned i = uround(t * 3600);
 
@@ -93,7 +94,8 @@ BreakHourOfDay(fixed t)
 }
 
 void
-WaypointInfoWidget::Prepare(ContainerWindow &parent, const PixelRect &rc)
+WaypointInfoWidget::Prepare(ContainerWindow &parent,
+                            const PixelRect &rc) noexcept
 {
   RowFormWidget::Prepare(parent, rc);
 
@@ -103,43 +105,42 @@ WaypointInfoWidget::Prepare(ContainerWindow &parent, const PixelRect &rc)
 
   StaticString<64> buffer;
 
-  if (!waypoint.comment.empty())
-    AddMultiLine(waypoint.comment.c_str());
+  if (!waypoint->comment.empty())
+    AddMultiLine(waypoint->comment.c_str());
 
-  if (waypoint.radio_frequency.IsDefined() &&
-      waypoint.radio_frequency.Format(buffer.buffer(),
+  if (waypoint->radio_frequency.IsDefined() &&
+      waypoint->radio_frequency.Format(buffer.buffer(),
                                       buffer.capacity()) != nullptr) {
     buffer += _T(" MHz");
     AddReadOnly(_("Radio frequency"), nullptr, buffer);
   }
 
-  if (waypoint.runway.IsDirectionDefined())
-    buffer.UnsafeFormat(_T("%02u"), waypoint.runway.GetDirectionName());
+  if (waypoint->runway.IsDirectionDefined())
+    buffer.UnsafeFormat(_T("%02u"), waypoint->runway.GetDirectionName());
   else
     buffer.clear();
 
-  if (waypoint.runway.IsLengthDefined()) {
+  if (waypoint->runway.IsLengthDefined()) {
     if (!buffer.empty())
       buffer += _T("; ");
 
     TCHAR length_buffer[16];
-    FormatSmallUserDistance(length_buffer,
-                                   fixed(waypoint.runway.GetLength()));
+    FormatSmallUserDistance(length_buffer, waypoint->runway.GetLength());
     buffer += length_buffer;
   }
 
   if (!buffer.empty())
     AddReadOnly(_("Runway"), nullptr, buffer);
 
-  if (FormatGeoPoint(waypoint.location,
+  if (FormatGeoPoint(waypoint->location,
                      buffer.buffer(), buffer.capacity()) != nullptr)
     AddReadOnly(_("Location"), nullptr, buffer);
 
-  AddReadOnly(_("Elevation"), nullptr, FormatUserAltitude(waypoint.elevation));
+  AddReadOnly(_("Elevation"), nullptr, FormatUserAltitude(waypoint->elevation));
 
   if (basic.time_available && basic.date_time_utc.IsDatePlausible()) {
     const SunEphemeris::Result sun =
-      SunEphemeris::CalcSunTimes(waypoint.location, basic.date_time_utc,
+      SunEphemeris::CalcSunTimes(waypoint->location, basic.date_time_utc,
                                  settings.utc_offset);
 
     const BrokenTime sunrise = BreakHourOfDay(sun.time_of_sunrise);
@@ -152,7 +153,7 @@ WaypointInfoWidget::Prepare(ContainerWindow &parent, const PixelRect &rc)
   }
 
   if (basic.location_available) {
-    const GeoVector vector = basic.location.DistanceBearing(waypoint.location);
+    const GeoVector vector = basic.location.DistanceBearing(waypoint->location);
 
     TCHAR distance_buffer[32];
     FormatUserDistanceSmart(vector.distance, distance_buffer,
@@ -165,13 +166,13 @@ WaypointInfoWidget::Prepare(ContainerWindow &parent, const PixelRect &rc)
 
   if (basic.location_available && basic.NavAltitudeAvailable() &&
       settings.polar.glide_polar_task.IsValid()) {
-    const GlideState glide_state(basic.location.DistanceBearing(waypoint.location),
-                                 waypoint.elevation + settings.task.safety_height_arrival,
+    const GlideState glide_state(basic.location.DistanceBearing(waypoint->location),
+                                 waypoint->elevation + settings.task.safety_height_arrival,
                                  basic.nav_altitude,
                                  calculated.GetWindOrZero());
 
     GlidePolar gp0 = settings.polar.glide_polar_task;
-    gp0.SetMC(fixed(0));
+    gp0.SetMC(0);
     AddGlideResult(_("Alt. diff. MC 0"),
                    MacCready::Solve(settings.task.glide,
                                     gp0, glide_state));
@@ -191,12 +192,12 @@ WaypointInfoWidget::Prepare(ContainerWindow &parent, const PixelRect &rc)
     const TaskBehaviour &task_behaviour =
       CommonInterface::GetComputerSettings().task;
 
-    const fixed safety_height = task_behaviour.safety_height_arrival;
-    const fixed target_altitude = waypoint.elevation + safety_height;
-    const fixed delta_h = basic.nav_altitude - target_altitude;
-    if (positive(delta_h)) {
-      const fixed distance = basic.location.Distance(waypoint.location);
-      const fixed gr = distance / delta_h;
+    const auto safety_height = task_behaviour.safety_height_arrival;
+    const auto target_altitude = waypoint->elevation + safety_height;
+    const auto delta_h = basic.nav_altitude - target_altitude;
+    if (delta_h > 0) {
+      const auto distance = basic.location.Distance(waypoint->location);
+      const auto gr = distance / delta_h;
       if (GradientValid(gr)) {
         buffer.UnsafeFormat(_T("%.1f"), (double)gr);
         AddReadOnly(_("Required glide ratio"), nullptr, buffer);
