@@ -1,7 +1,7 @@
 /* Copyright_License {
 
   XCSoar Glide Computer - http://www.xcsoar.org/
-  Copyright (C) 2000-2015 The XCSoar Project
+  Copyright (C) 2000-2021 The XCSoar Project
   A detailed list of copyright holders can be found in the file "AUTHORS".
 
   This program is free software; you can redistribute it and/or
@@ -24,7 +24,7 @@
 #define XCSOAR_WINDOW_STATS_COMPUTER_HPP
 
 #include "Math/DifferentialWindowFilter.hpp"
-#include "Time/DeltaTime.hpp"
+#include "time/DeltaTime.hpp"
 
 struct WindowStats;
 class TaskStats;
@@ -40,7 +40,7 @@ public:
     minute_clock.Reset();
   }
 
-  void Compute(fixed time, const TaskStats &task_stats, WindowStats &stats);
+  void Compute(double time, const TaskStats &task_stats, WindowStats &stats);
 };
 
 #endif
