@@ -1,7 +1,7 @@
 /* Copyright_License {
 
   XCSoar Glide Computer - http://www.xcsoar.org/
-  Copyright (C) 2000-2015 The XCSoar Project
+  Copyright (C) 2000-2021 The XCSoar Project
   A detailed list of copyright holders can be found in the file "AUTHORS".
 
   This program is free software; you can redistribute it and/or
@@ -22,7 +22,6 @@
 
 #include "OrderedTask.hpp"
 #include "Task/TaskEvents.hpp"
-#include "TaskAdvance.hpp"
 #include "Points/OrderedTaskPoint.hpp"
 #include "Points/StartPoint.hpp"
 #include "Points/FinishPoint.hpp"
@@ -65,48 +64,39 @@ constexpr bool subtract_start_finish_cylinder_radius = true;
  * Determine the cylinder radius if this is a CylinderZone.  If not,
  * return -1.
  */
-gcc_pure
-static fixed
+[[gnu::pure]]
+static double
 GetCylinderRadiusOrMinusOne(const ObservationZone &oz)
 {
   return oz.GetShape() == ObservationZone::Shape::CYLINDER
     ? ((const CylinderZone &)oz).GetRadius()
-    : fixed(-1);
+    : -1;
 }
 
 /**
  * Determine the cylinder radius if this is a CylinderZone.  If not,
  * return -1.
  */
-gcc_pure
-static fixed
+[[gnu::pure]]
+static double
 GetCylinderRadiusOrMinusOne(const ObservationZoneClient &p)
 {
   return GetCylinderRadiusOrMinusOne(p.GetObservationZone());
 }
 
-OrderedTask::OrderedTask(const TaskBehaviour &tb)
+OrderedTask::OrderedTask(const TaskBehaviour &tb) noexcept
   :AbstractTask(TaskType::ORDERED, tb),
-   taskpoint_start(nullptr),
-   taskpoint_finish(nullptr),
    factory_mode(tb.task_type_default),
-   active_factory(nullptr),
-   ordered_settings(tb.ordered_defaults),
-   dijkstra_min(nullptr), dijkstra_max(nullptr)
+   ordered_settings(tb.ordered_defaults)
 {
   ClearName();
   active_factory = CreateTaskFactory(factory_mode, *this, task_behaviour);
   active_factory->UpdateOrderedTaskSettings(ordered_settings);
 }
 
-OrderedTask::~OrderedTask()
+OrderedTask::~OrderedTask() noexcept
 {
   RemoveAllPoints();
-
-  delete active_factory;
-
-  delete dijkstra_min;
-  delete dijkstra_max;
 }
 
 const TaskFactoryConstraints &
@@ -119,7 +109,7 @@ static void
 SetTaskBehaviour(OrderedTask::OrderedTaskPointVector &vector,
                  const TaskBehaviour &tb)
 {
-  for (auto i : vector)
+  for (const auto &i : vector)
     i->SetTaskBehaviour(tb);
 }
 
@@ -136,7 +126,7 @@ static void
 UpdateObservationZones(OrderedTask::OrderedTaskPointVector &points,
                        const FlatProjection &projection)
 {
-  for (auto i : points)
+  for (const auto &i : points)
     i->UpdateOZ(projection);
 }
 
@@ -145,7 +135,7 @@ OrderedTask::UpdateStatsGeometry()
 {
   ScanStartFinish();
 
-  stats.task_valid = CheckTask();
+  stats.task_valid = !IsError(CheckTask());
   stats.has_targets = stats.task_valid && HasTargets();
   stats.is_mat = GetFactoryType() == TaskFactoryType::MAT;
   stats.has_optional_starts = stats.task_valid && HasOptionalStarts();
@@ -165,11 +155,11 @@ OrderedTask::UpdateGeometry()
 
   // scan location of task points
   GeoBounds bounds(first.GetLocation());
-  for (const auto *tp : task_points)
+  for (const auto &tp : task_points)
     tp->ScanBounds(bounds);
 
   // ... and optional start points
-  for (const OrderedTaskPoint *tp : optional_start_points)
+  for (const auto &tp : optional_start_points)
     tp->ScanBounds(bounds);
 
   // projection can now be determined
@@ -181,10 +171,10 @@ OrderedTask::UpdateGeometry()
 
   // now that the task projection is stable, and oz is stable,
   // calculate the bounding box in projected coordinates
-  for (const auto tp : task_points)
+  for (const auto &tp : task_points)
     tp->UpdateBoundingBox(task_projection);
 
-  for (const auto tp : optional_start_points)
+  for (const auto &tp : optional_start_points)
     tp->UpdateBoundingBox(task_projection);
 
   // update stats so data can be used during task construction
@@ -202,22 +192,22 @@ OrderedTask::UpdateGeometry()
 
 // TIMES
 
-fixed
-OrderedTask::ScanTotalStartTime()
+double
+OrderedTask::ScanTotalStartTime() noexcept
 {
   if (task_points.empty())
-    return fixed(-1);
+    return -1;
 
   return task_points.front()->GetEnteredState().time;
 }
 
-fixed
-OrderedTask::ScanLegStartTime()
+double
+OrderedTask::ScanLegStartTime() noexcept
 {
   if (active_task_point > 0)
     return task_points[active_task_point-1]->GetEnteredState().time;
 
-  return fixed(-1);
+  return -1;
 }
 
 // DISTANCES
@@ -230,7 +220,7 @@ OrderedTask::RunDijsktraMin(const GeoPoint &location)
     return false;
 
   if (dijkstra_min == nullptr)
-    dijkstra_min = new TaskDijkstraMin();
+    dijkstra_min = std::make_unique<TaskDijkstraMin>();
   TaskDijkstraMin &dijkstra = *dijkstra_min;
 
   const unsigned active_index = GetActiveIndex();
@@ -250,7 +240,7 @@ OrderedTask::RunDijsktraMin(const GeoPoint &location)
   return true;
 }
 
-inline fixed
+inline double
 OrderedTask::ScanDistanceMin(const GeoPoint &location, bool full)
 {
   if (!full && location.IsValid() && last_min_location.IsValid() &&
@@ -289,7 +279,7 @@ OrderedTask::RunDijsktraMax()
     return false;
 
   if (dijkstra_max == nullptr)
-    dijkstra_max = new TaskDijkstraMax();
+    dijkstra_max = std::make_unique<TaskDijkstraMax>();
   TaskDijkstraMax &dijkstra = *dijkstra_max;
 
   const unsigned active_index = GetActiveIndex();
@@ -303,7 +293,7 @@ OrderedTask::RunDijsktraMax()
     dijkstra_max->SetBoundary(i, boundary);
   }
 
-  fixed start_radius(-1), finish_radius(-1);
+  double start_radius(-1), finish_radius(-1);
   if (subtract_start_finish_cylinder_radius) {
     /* to subtract the start/finish cylinder radius, we use only the
        nominal points (i.e. the cylinder's center), and later replace
@@ -311,12 +301,12 @@ OrderedTask::RunDijsktraMax()
 
     const auto &start = *task_points.front();
     start_radius = GetCylinderRadiusOrMinusOne(start);
-    if (positive(start_radius))
+    if (start_radius > 0)
       dijkstra.SetBoundary(0, start.GetNominalPoints());
 
     const auto &finish = *task_points.back();
     finish_radius = GetCylinderRadiusOrMinusOne(finish);
-    if (positive(finish_radius))
+    if (finish_radius > 0)
       dijkstra.SetBoundary(task_size - 1, finish.GetNominalPoints());
   }
 
@@ -326,7 +316,7 @@ OrderedTask::RunDijsktraMax()
   for (unsigned i = 0; i != task_size; ++i) {
     SearchPoint solution = dijkstra.GetSolution(i);
 
-    if (i == 0 && positive(start_radius)) {
+    if (i == 0 && start_radius > 0) {
       /* subtract start cylinder radius by finding the intersection
          with the cylinder boundary */
       const GeoPoint &current = task_points.front()->GetLocation();
@@ -335,7 +325,7 @@ OrderedTask::RunDijsktraMax()
       solution = SearchPoint(gp, task_projection);
     }
 
-    if (i == task_size - 1 && positive(finish_radius)) {
+    if (i == task_size - 1 && finish_radius > 0) {
       /* subtract finish cylinder radius by finding the intersection
          with the cylinder boundary */
       const GeoPoint &current = task_points.back()->GetLocation();
@@ -352,11 +342,11 @@ OrderedTask::RunDijsktraMax()
   return true;
 }
 
-inline fixed
+inline double
 OrderedTask::ScanDistanceMax()
 {
   if (task_points.empty()) // nothing to do!
-    return fixed(0);
+    return 0;
 
   assert(active_task_point < task_points.size());
 
@@ -367,7 +357,7 @@ OrderedTask::ScanDistanceMax()
 
 void
 OrderedTask::ScanDistanceMinMax(const GeoPoint &location, bool force,
-                                fixed *dmin, fixed *dmax)
+                                double *dmin, double *dmax) noexcept
 {
   if (force)
     *dmax = ScanDistanceMax();
@@ -375,56 +365,56 @@ OrderedTask::ScanDistanceMinMax(const GeoPoint &location, bool force,
   *dmin = ScanDistanceMin(location, force);
 }
 
-fixed
-OrderedTask::ScanDistanceNominal()
+double
+OrderedTask::ScanDistanceNominal() noexcept
 {
   if (task_points.empty())
-    return fixed(0);
+    return 0;
 
   const auto &start = *task_points.front();
-  fixed d = start.ScanDistanceNominal();
+  auto d = start.ScanDistanceNominal();
 
-  fixed radius = GetCylinderRadiusOrMinusOne(start);
-  if (positive(radius) && radius < d)
+  auto radius = GetCylinderRadiusOrMinusOne(start);
+  if (radius > 0 && radius < d)
     d -= radius;
 
   const auto &finish = *task_points.back();
   radius = GetCylinderRadiusOrMinusOne(finish);
-  if (positive(radius) && radius < d)
+  if (radius > 0 && radius < d)
     d -= radius;
 
   return d;
 }
 
-fixed
-OrderedTask::ScanDistanceScored(const GeoPoint &location)
+double
+OrderedTask::ScanDistanceScored(const GeoPoint &location) noexcept
 {
   return task_points.empty()
-    ? fixed(0)
+    ? 0
     : task_points.front()->ScanDistanceScored(location);
 }
 
-fixed
-OrderedTask::ScanDistanceRemaining(const GeoPoint &location)
+double
+OrderedTask::ScanDistanceRemaining(const GeoPoint &location) noexcept
 {
   return task_points.empty()
-    ? fixed(0)
+    ? 0
     : task_points.front()->ScanDistanceRemaining(location);
 }
 
-fixed
-OrderedTask::ScanDistanceTravelled(const GeoPoint &location)
+double
+OrderedTask::ScanDistanceTravelled(const GeoPoint &location) noexcept
 {
   return task_points.empty()
-    ? fixed(0)
+    ? 0
     : task_points.front()->ScanDistanceTravelled(location);
 }
 
-fixed
-OrderedTask::ScanDistancePlanned()
+double
+OrderedTask::ScanDistancePlanned() noexcept
 {
   return task_points.empty()
-    ? fixed(0)
+    ? 0
     : task_points.front()->ScanDistancePlanned();
 }
 
@@ -444,7 +434,7 @@ OrderedTask::GetLastIntermediateAchieved() const
 
 bool
 OrderedTask::CheckTransitions(const AircraftState &state,
-                              const AircraftState &state_last)
+                              const AircraftState &state_last) noexcept
 {
   if (!taskpoint_start)
     return false;
@@ -529,7 +519,7 @@ OrderedTask::CheckTransitions(const AircraftState &state,
     stats.start.SetStarted(start_state);
 
     if (taskpoint_finish != nullptr)
-      taskpoint_finish->set_fai_finish_height(start_state.altitude - fixed(1000));
+      taskpoint_finish->SetFaiFinishHeight(start_state.altitude - 1000);
   }
 
   if (task_events != nullptr) {
@@ -618,24 +608,25 @@ OrderedTask::CheckTransitionPoint(OrderedTaskPoint &point,
 
 bool
 OrderedTask::UpdateIdle(const AircraftState &state,
-                        const GlidePolar &glide_polar)
+                        const GlidePolar &glide_polar) noexcept
 {
   bool retval = AbstractTask::UpdateIdle(state, glide_polar);
 
   if (HasStart() && task_behaviour.optimise_targets_range &&
-      positive(GetOrderedTaskSettings().aat_min_time)) {
+      GetOrderedTaskSettings().aat_min_time > 0) {
 
     CalcMinTarget(state, glide_polar,
-                  GetOrderedTaskSettings().aat_min_time + fixed(task_behaviour.optimise_targets_margin));
+                  GetOrderedTaskSettings().aat_min_time + task_behaviour.optimise_targets_margin);
 
     if (task_behaviour.optimise_targets_bearing &&
         task_points[active_task_point]->GetType() == TaskPointType::AAT) {
-      AATPoint *ap = (AATPoint *)task_points[active_task_point];
+      TaskPointList tps(task_points);
+      AATPoint *ap = (AATPoint *)task_points[active_task_point].get();
       // very nasty hack
-      TaskOptTarget tot(task_points, active_task_point, state,
+      TaskOptTarget tot(tps, active_task_point, state,
                         task_behaviour.glide, glide_polar,
-                        *ap, task_projection, taskpoint_start);
-      tot.search(fixed(0.5));
+                        *ap, task_projection, *taskpoint_start);
+      tot.search(0.5);
     }
     retval = true;
   }
@@ -646,7 +637,7 @@ OrderedTask::UpdateIdle(const AircraftState &state,
 bool
 OrderedTask::UpdateSample(const AircraftState &state,
                           gcc_unused const GlidePolar &glide_polar,
-                          gcc_unused const bool full_update)
+                          gcc_unused const bool full_update) noexcept
 {
   assert(state.location.IsValid());
 
@@ -669,21 +660,21 @@ OrderedTask::SetNeighbours(unsigned position)
     return;
 
   if (position > 0)
-    prev = task_points[position - 1];
+    prev = task_points[position - 1].get();
 
   if (position + 1 < task_points.size())
-    next = task_points[position + 1];
+    next = task_points[position + 1].get();
 
   task_points[position]->SetNeighbours(prev, next);
 
   if (position==0) {
-    for (const auto tp : optional_start_points)
+    for (const auto &tp : optional_start_points)
       tp->SetNeighbours(prev, next);
   }
 }
 
-bool
-OrderedTask::CheckTask() const
+TaskValidationErrorSet
+OrderedTask::CheckTask() const noexcept
 {
   return this->GetFactory().Validate();
 }
@@ -696,7 +687,7 @@ OrderedTask::GetAATTaskPoint(unsigned TPindex) const
  }
 
  if (task_points[TPindex]->GetType() == TaskPointType::AAT)
-   return (AATPoint *)task_points[TPindex];
+   return (AATPoint *)task_points[TPindex].get();
  else
    return (AATPoint *)nullptr;
 }
@@ -712,12 +703,12 @@ OrderedTask::ScanStartFinish()
   }
 
   taskpoint_start = task_points.front()->GetType() == TaskPointType::START
-    ? (StartPoint *)task_points.front()
+    ? (StartPoint *)task_points.front().get()
     : nullptr;
 
   taskpoint_finish = task_points.size() > 1 &&
     task_points.back()->GetType() == TaskPointType::FINISH
-    ? (FinishPoint *)task_points.back()
+    ? (FinishPoint *)task_points.back().get()
     : nullptr;
 
   return HasStart() && HasFinish();
@@ -726,14 +717,12 @@ OrderedTask::ScanStartFinish()
 inline void
 OrderedTask::ErasePoint(const unsigned index)
 {
-  delete task_points[index];
   task_points.erase(task_points.begin() + index);
 }
 
 inline void
 OrderedTask::EraseOptionalStartPoint(const unsigned index)
 {
-  delete optional_start_points[index];
   optional_start_points.erase(optional_start_points.begin() + index);
 }
 
@@ -783,7 +772,7 @@ OrderedTask::Append(const OrderedTaskPoint &new_tp)
     return false;
 
   const unsigned i = task_points.size();
-  task_points.push_back(new_tp.Clone(task_behaviour, ordered_settings));
+  task_points.emplace_back(new_tp.Clone(task_behaviour, ordered_settings));
   if (i > 0)
     SetNeighbours(i - 1);
   else {
@@ -798,8 +787,8 @@ OrderedTask::Append(const OrderedTaskPoint &new_tp)
 bool
 OrderedTask::AppendOptionalStart(const OrderedTaskPoint &new_tp)
 {
-  optional_start_points.push_back(new_tp.Clone(task_behaviour,
-                                               ordered_settings));
+  optional_start_points.emplace_back(new_tp.Clone(task_behaviour,
+                                                  ordered_settings));
   if (task_points.size() > 1)
     SetNeighbours(0);
   return true;
@@ -849,7 +838,6 @@ OrderedTask::Replace(const OrderedTaskPoint &new_tp, const unsigned position)
       (position + 1 < task_points.size() && !new_tp.IsSuccessorAllowed()))
     return false;
 
-  delete task_points[position];
   task_points[position] = new_tp.Clone(task_behaviour, ordered_settings);
 
   if (position)
@@ -874,7 +862,6 @@ OrderedTask::ReplaceOptionalStart(const OrderedTaskPoint &new_tp,
     // nothing to do
     return true;
 
-  delete optional_start_points[position];
   optional_start_points[position] = new_tp.Clone(task_behaviour,
                                                  ordered_settings);
 
@@ -884,7 +871,7 @@ OrderedTask::ReplaceOptionalStart(const OrderedTaskPoint &new_tp,
 
 
 void
-OrderedTask::SetActiveTaskPoint(unsigned index)
+OrderedTask::SetActiveTaskPoint(unsigned index) noexcept
 {
   if (index >= task_points.size() || index == active_task_point)
     return;
@@ -895,16 +882,16 @@ OrderedTask::SetActiveTaskPoint(unsigned index)
 }
 
 TaskWaypoint*
-OrderedTask::GetActiveTaskPoint() const
+OrderedTask::GetActiveTaskPoint() const noexcept
 {
   if (active_task_point < task_points.size())
-    return task_points[active_task_point];
+    return task_points[active_task_point].get();
 
   return nullptr;
 }
 
 bool
-OrderedTask::IsValidTaskPoint(const int index_offset) const
+OrderedTask::IsValidTaskPoint(const int index_offset) const noexcept
 {
   unsigned index = active_task_point + index_offset;
   return (index < task_points.size());
@@ -914,7 +901,7 @@ void
 OrderedTask::GlideSolutionRemaining(const AircraftState &aircraft,
                                     const GlidePolar &polar,
                                     GlideResult &total,
-                                    GlideResult &leg)
+                                    GlideResult &leg) noexcept
 {
   if (!aircraft.location.IsValid() || task_points.empty()) {
     total.Reset();
@@ -922,7 +909,8 @@ OrderedTask::GlideSolutionRemaining(const AircraftState &aircraft,
     return;
   }
 
-  TaskMacCreadyRemaining tm(task_points.cbegin(), task_points.cend(),
+  TaskPointList tps(task_points);
+  TaskMacCreadyRemaining tm(tps.begin(), tps.end(),
                             active_task_point,
                             task_behaviour.glide, polar);
   total = tm.glide_solution(aircraft);
@@ -933,7 +921,7 @@ void
 OrderedTask::GlideSolutionTravelled(const AircraftState &aircraft,
                                     const GlidePolar &glide_polar,
                                     GlideResult &total,
-                                    GlideResult &leg)
+                                    GlideResult &leg) noexcept
 {
   if (!aircraft.location.IsValid() || task_points.empty()) {
     total.Reset();
@@ -941,7 +929,8 @@ OrderedTask::GlideSolutionTravelled(const AircraftState &aircraft,
     return;
   }
 
-  TaskMacCreadyTravelled tm(task_points.cbegin(), active_task_point,
+  TaskPointList tps(task_points);
+  TaskMacCreadyTravelled tm(tps.begin(), active_task_point,
                             task_behaviour.glide, glide_polar);
   total = tm.glide_solution(aircraft);
   leg = tm.get_active_solution();
@@ -955,7 +944,7 @@ OrderedTask::GlideSolutionPlanned(const AircraftState &aircraft,
                                   DistanceStat &total_remaining_effective,
                                   DistanceStat &leg_remaining_effective,
                                   const GlideResult &solution_remaining_total,
-                                  const GlideResult &solution_remaining_leg)
+                                  const GlideResult &solution_remaining_leg) noexcept
 {
   if (task_points.empty()) {
     total.Reset();
@@ -965,7 +954,8 @@ OrderedTask::GlideSolutionPlanned(const AircraftState &aircraft,
     return;
   }
 
-  TaskMacCreadyTotal tm(task_points.cbegin(), task_points.cend(),
+  TaskPointList tps(task_points);
+  TaskMacCreadyTotal tm(tps.begin(), tps.end(),
                         active_task_point,
                         task_behaviour.glide, glide_polar);
   total = tm.glide_solution(aircraft);
@@ -984,22 +974,24 @@ OrderedTask::GlideSolutionPlanned(const AircraftState &aircraft,
 
 // Auxiliary glide functions
 
-fixed
+double
 OrderedTask::CalcRequiredGlide(const AircraftState &aircraft,
-                               const GlidePolar &glide_polar) const
+                               const GlidePolar &glide_polar) const noexcept
 {
-  TaskGlideRequired bgr(task_points, active_task_point, aircraft,
+  TaskPointList tps(task_points);
+  TaskGlideRequired bgr(tps, active_task_point, aircraft,
                         task_behaviour.glide, glide_polar);
-  return bgr.search(fixed(0));
+  return bgr.search(0);
 }
 
 bool
 OrderedTask::CalcBestMC(const AircraftState &aircraft,
                         const GlidePolar &glide_polar,
-                        fixed &best) const
+                        double &best) const noexcept
 {
   // note setting of lower limit on mc
-  TaskBestMc bmc(task_points, active_task_point, aircraft,
+  TaskPointList tps(task_points);
+  TaskBestMc bmc(tps, active_task_point, aircraft,
                  task_behaviour.glide, glide_polar);
   return bmc.search(glide_polar.GetMC(), best);
 }
@@ -1024,15 +1016,16 @@ OrderedTask::AllowIncrementalBoundaryStats(const AircraftState &aircraft) const
 bool
 OrderedTask::CalcCruiseEfficiency(const AircraftState &aircraft,
                                   const GlidePolar &glide_polar,
-                                  fixed &val) const
+                                  double &val) const noexcept
 {
   if (AllowIncrementalBoundaryStats(aircraft)) {
-    TaskCruiseEfficiency bce(task_points, active_task_point, aircraft,
+    TaskPointList tps(task_points);
+    TaskCruiseEfficiency bce(tps, active_task_point, aircraft,
                              task_behaviour.glide, glide_polar);
-    val = bce.search(fixed(1));
+    val = bce.search(1);
     return true;
   } else {
-    val = fixed(1);
+    val = 1;
     return false;
   }
 }
@@ -1040,10 +1033,11 @@ OrderedTask::CalcCruiseEfficiency(const AircraftState &aircraft,
 bool
 OrderedTask::CalcEffectiveMC(const AircraftState &aircraft,
                              const GlidePolar &glide_polar,
-                             fixed &val) const
+                             double &val) const noexcept
 {
   if (AllowIncrementalBoundaryStats(aircraft)) {
-    TaskEffectiveMacCready bce(task_points, active_task_point, aircraft,
+    TaskPointList tps(task_points);
+    TaskEffectiveMacCready bce(tps, active_task_point, aircraft,
                                task_behaviour.glide, glide_polar);
     val = bce.search(glide_polar.GetMC());
     return true;
@@ -1054,40 +1048,40 @@ OrderedTask::CalcEffectiveMC(const AircraftState &aircraft,
 }
 
 
-inline fixed
+inline double
 OrderedTask::CalcMinTarget(const AircraftState &aircraft,
                            const GlidePolar &glide_polar,
-                           const fixed t_target)
+                           const double t_target)
 {
   if (stats.has_targets) {
     // only perform scan if modification is possible
-    const fixed t_rem = std::max(fixed(0),
-                                 t_target - stats.total.time_elapsed);
+    const auto t_rem = fdim(t_target, stats.total.time_elapsed);
 
-    TaskMinTarget bmt(task_points, active_task_point, aircraft,
+    TaskPointList tps(task_points);
+    TaskMinTarget bmt(tps, active_task_point, aircraft,
                       task_behaviour.glide, glide_polar,
-                      t_rem, taskpoint_start);
-    fixed p = bmt.search(fixed(0));
+                      t_rem, *taskpoint_start);
+    auto p = bmt.search(0);
     return p;
   }
 
-  return fixed(0);
+  return 0;
 }
 
-fixed
-OrderedTask::CalcGradient(const AircraftState &state) const
+double
+OrderedTask::CalcGradient(const AircraftState &state) const noexcept
 {
   if (task_points.empty())
-    return fixed(0);
+    return 0;
 
   // Iterate through remaining turnpoints
-  fixed distance = fixed(0);
-  for (const OrderedTaskPoint *tp : task_points)
+  double distance = 0;
+  for (const auto &tp : task_points)
     // Sum up the leg distances
     distance += tp->GetVectorRemaining(state.location).distance;
 
-  if (!positive(distance))
-    return fixed(0);
+  if (distance <= 0)
+    return 0;
 
   // Calculate gradient to the last turnpoint of the remaining task
   return (state.altitude - task_points.back()->GetElevation()) / distance;
@@ -1097,7 +1091,7 @@ static void
 Visit(const OrderedTask::OrderedTaskPointVector &points,
       TaskPointConstVisitor &visitor)
 {
-  for (const TaskPoint *tp : points)
+  for (const auto &tp : points)
     visitor.Visit(*tp);
 }
 
@@ -1110,12 +1104,12 @@ OrderedTask::AcceptTaskPointVisitor(TaskPointConstVisitor& visitor) const
 static void
 ResetPoints(OrderedTask::OrderedTaskPointVector &points)
 {
-  for (auto *i : points)
+  for (auto &i : points)
     i->Reset();
 }
 
 void
-OrderedTask::Reset()
+OrderedTask::Reset() noexcept
 {
   /// @todo also reset data in this class e.g. stats?
   ResetPoints(task_points);
@@ -1130,7 +1124,7 @@ OrderedTask::Reset()
 }
 
 bool
-OrderedTask::TaskStarted(bool soft) const
+OrderedTask::TaskStarted(bool soft) const noexcept
 {
   if (taskpoint_start) {
     // have we really started?
@@ -1155,7 +1149,7 @@ OrderedTask::TaskStarted(bool soft) const
  *
  * @return True if distance is significant
  */
-gcc_pure
+[[gnu::pure]]
 static bool
 DistanceIsSignificant(const SearchPoint &a1, const SearchPoint &a2,
                       const unsigned dist_threshold = 1)
@@ -1201,7 +1195,7 @@ OrderedTask::SetPointSearchMax(unsigned tp, const SearchPoint &sol)
 bool
 OrderedTask::IsFull() const
 {
-  return TaskSize() == GetFactory().GetConstraints().max_points;
+  return TaskSize() >= GetFactory().GetConstraints().max_points;
 }
 
 inline void
@@ -1223,26 +1217,26 @@ OrderedTask::UpdateStartTransition(const AircraftState &state,
 bool
 OrderedTask::HasTargets() const
 {
-  for (const OrderedTaskPoint *tp : task_points)
+  for (const auto &tp : task_points)
     if (tp->HasTarget())
       return true;
 
   return false;
 }
 
-OrderedTask*
-OrderedTask::Clone(const TaskBehaviour &tb) const
+std::unique_ptr<OrderedTask>
+OrderedTask::Clone(const TaskBehaviour &tb) const noexcept
 {
-  OrderedTask* new_task = new OrderedTask(tb);
+  auto new_task = std::make_unique<OrderedTask>(tb);
 
   new_task->SetFactory(factory_mode);
 
   new_task->ordered_settings = ordered_settings;
 
-  for (const OrderedTaskPoint *tp : task_points)
+  for (const auto &tp : task_points)
     new_task->Append(*tp);
 
-  for (const OrderedTaskPoint *tp : optional_start_points)
+  for (const auto &tp : optional_start_points)
     new_task->AppendOptionalStart(*tp);
 
   new_task->active_task_point = active_task_point;
@@ -1260,16 +1254,14 @@ OrderedTask::CheckDuplicateWaypoints(Waypoints& waypoints,
 {
   for (auto begin = points.cbegin(), end = points.cend(), i = begin;
        i != end; ++i) {
-    const Waypoint &wp =
-      waypoints.CheckExistsOrAppend((*i)->GetWaypoint());
+    auto wp = waypoints.CheckExistsOrAppend((*i)->GetWaypointPtr());
 
-    const OrderedTaskPoint *new_tp =
-      (*i)->Clone(task_behaviour, ordered_settings, &wp);
+    const auto new_tp =
+      (*i)->Clone(task_behaviour, ordered_settings, std::move(wp));
     if (is_task)
       Replace(*new_tp, std::distance(begin, i));
     else
       ReplaceOptionalStart(*new_tp, std::distance(begin, i));
-    delete new_tp;
   }
 }
 
@@ -1340,30 +1332,28 @@ OrderedTask::Commit(const OrderedTask& that)
 }
 
 bool
-OrderedTask::RelocateOptionalStart(const unsigned position, const Waypoint& waypoint)
+OrderedTask::RelocateOptionalStart(const unsigned position,
+                                   WaypointPtr &&waypoint)
 {
   if (position >= optional_start_points.size())
     return false;
 
-  OrderedTaskPoint *new_tp =
+  optional_start_points[position] =
     optional_start_points[position]->Clone(task_behaviour, ordered_settings,
-                                           &waypoint);
-  delete optional_start_points[position];
-  optional_start_points[position]= new_tp;
+                                           std::move(waypoint));
   return true;
 }
 
 bool
-OrderedTask::Relocate(const unsigned position, const Waypoint& waypoint)
+OrderedTask::Relocate(const unsigned position, WaypointPtr &&waypoint)
 {
   if (position >= TaskSize())
     return false;
 
-  OrderedTaskPoint *new_tp = task_points[position]->Clone(task_behaviour,
-                                                  ordered_settings,
-                                                  &waypoint);
+  auto new_tp = task_points[position]->Clone(task_behaviour,
+                                             ordered_settings,
+                                             std::move(waypoint));
   bool success = Replace(*new_tp, position);
-  delete new_tp;
   return success;
 }
 
@@ -1383,7 +1373,6 @@ OrderedTask::SetFactory(const TaskFactoryType the_factory)
   }
   factory_mode = the_factory;
 
-  delete active_factory;
   active_factory = CreateTaskFactory(factory_mode, *this, task_behaviour);
   active_factory->UpdateOrderedTaskSettings(ordered_settings);
 
@@ -1401,15 +1390,15 @@ OrderedTask::SetOrderedTaskSettings(const OrderedTaskSettings& ob)
 void
 OrderedTask::PropagateOrderedTaskSettings()
 {
-  for (auto tp : task_points)
+  for (auto &tp : task_points)
     tp->SetOrderedTaskSettings(ordered_settings);
 
-  for (auto tp : optional_start_points)
+  for (auto &tp : optional_start_points)
     tp->SetOrderedTaskSettings(ordered_settings);
 }
 
 bool
-OrderedTask::IsScored() const
+OrderedTask::IsScored() const noexcept
 {
   return GetFactoryConstraints().task_scored;
 }
@@ -1429,14 +1418,7 @@ OrderedTask::GetFactoryTypes(gcc_unused bool all) const
 void
 OrderedTask::RemoveAllPoints()
 {
-  for (auto i : task_points)
-    delete i;
-
   task_points.clear();
-
-  for (auto i : optional_start_points)
-    delete i;
-
   optional_start_points.clear();
 
   active_task_point = 0;
@@ -1457,28 +1439,6 @@ OrderedTask::Clear()
   active_factory->UpdateOrderedTaskSettings(ordered_settings);
 }
 
-FlatBoundingBox
-OrderedTask::GetBoundingBox(const GeoBounds &bounds) const
-{
-  if (IsEmpty()) {
-    // undefined!
-    return FlatBoundingBox(FlatGeoPoint(0,0),FlatGeoPoint(0,0));
-  }
-
-  FlatGeoPoint ll = task_projection.ProjectInteger(bounds.GetSouthWest());
-  FlatGeoPoint lr = task_projection.ProjectInteger(bounds.GetSouthEast());
-  FlatGeoPoint ul = task_projection.ProjectInteger(bounds.GetNorthWest());
-  FlatGeoPoint ur = task_projection.ProjectInteger(bounds.GetNorthEast());
-  FlatGeoPoint fmin(std::min(ll.longitude, ul.longitude),
-                    std::min(ll.latitude, lr.latitude));
-  FlatGeoPoint fmax(std::max(lr.longitude, ur.longitude),
-                    std::max(ul.latitude, ur.latitude));
-  // note +/- 1 to ensure rounding keeps bb valid
-  fmin.longitude -= 1; fmin.latitude -= 1;
-  fmax.longitude += 1; fmax.latitude += 1;
-  return FlatBoundingBox (fmin, fmax);
-}
-
 void
 OrderedTask::RotateOptionalStarts()
 {
@@ -1494,9 +1454,9 @@ OrderedTask::SelectOptionalStart(unsigned pos)
   assert(pos< optional_start_points.size());
 
   // put task start onto end
-  optional_start_points.push_back(task_points.front());
+  optional_start_points.push_back(std::move(task_points.front()));
   // set task start from top optional item
-  task_points.front() = optional_start_points[pos];
+  task_points.front() = std::move(optional_start_points[pos]);
   // remove top optional item from list
   optional_start_points.erase(optional_start_points.begin()+pos);
 
@@ -1517,7 +1477,7 @@ OrderedTask::UpdateSummary(TaskSummary& ordered_summary) const
   ordered_summary.active = active_task_point;
 
   bool first = true;
-  for (const auto *tpp : task_points) {
+  for (const auto &tpp : task_points) {
     const OrderedTaskPoint &tp = *tpp;
 
     TaskSummaryPoint tsp;
