@@ -2,7 +2,7 @@
 Copyright_License {
 
   XCSoar Glide Computer - http://www.xcsoar.org/
-  Copyright (C) 2000-2015 The XCSoar Project
+  Copyright (C) 2000-2021 The XCSoar Project
   A detailed list of copyright holders can be found in the file "AUTHORS".
 
   This program is free software; you can redistribute it and/or
@@ -24,11 +24,11 @@ Copyright_License {
 #ifndef XCSOAR_INFO_BOX_SETTINGS_HPP
 #define XCSOAR_INFO_BOX_SETTINGS_HPP
 
-#include "Util/StaticString.hxx"
-#include "Compiler.h"
+#include "util/StaticString.hxx"
+#include "util/Compiler.h"
 #include "InfoBoxes/Content/Type.hpp"
 
-#include <stdint.h>
+#include <cstdint>
 
 struct InfoBoxSettings {
   enum PanelIndex {
@@ -79,7 +79,7 @@ struct InfoBoxSettings {
     /** @see #BOTTOM_RIGHT_8 */
     OBSOLETE_BOTTOM_RIGHT_8 = 5,
 
-    /** infoboxes GNAV (9 right + vario) */
+    /** 9 right + vario */
     RIGHT_9_VARIO = 6,
 
     /** infoboxes (5) along right side (square screen) */
@@ -113,6 +113,16 @@ struct InfoBoxSettings {
 
     /** 16 infoboxes along right side (2x8) */
     RIGHT_16 = 19,
+    LEFT_12_RIGHT_3_VARIO = 20,
+
+    /** 10 infoboxes along top or left */
+    TOP_LEFT_10 = 21,
+    /** 10 infoboxes along bottom or right side */
+    BOTTOM_RIGHT_10 = 22,
+    /** 10 infoboxes split bottom/top or left/right */
+    SPLIT_10 = 23,
+    /** 12 infoboxes 3X4 split bottom/top or left/right */
+    SPLIT_3X4 = 24,
 
   } geometry;
 
