@@ -2,7 +2,7 @@
 Copyright_License {
 
   XCSoar Glide Computer - http://www.xcsoar.org/
-  Copyright (C) 2000-2015 The XCSoar Project
+  Copyright (C) 2000-2021 The XCSoar Project
   A detailed list of copyright holders can be found in the file "AUTHORS".
 
   This program is free software; you can redistribute it and/or
@@ -25,50 +25,35 @@ Copyright_License {
 #include "InfoBoxSettings.hpp"
 #include "Border.hpp"
 #include "Look/InfoBoxLook.hpp"
-#include "Look/UnitsLook.hpp"
 #include "Input/InputEvents.hpp"
 #include "Renderer/GlassRenderer.hpp"
 #include "Renderer/UnitSymbolRenderer.hpp"
-#include "Screen/UnitSymbol.hpp"
 #include "Screen/Layout.hpp"
-#include "Screen/BufferCanvas.hpp"
-#include "Screen/Key.h"
+#include "ui/canvas/Canvas.hpp"
+#include "ui/event/KeyCode.hpp"
 #include "Dialogs/dlgInfoBoxAccess.hpp"
-
-#ifdef ENABLE_OPENGL
-#include "Screen/SubCanvas.hpp"
-#endif
+#include "Asset.hpp"
 
 #include <algorithm>
 
-using std::max;
+/** timeout of infobox focus */
+static constexpr std::chrono::steady_clock::duration FOCUS_TIMEOUT_MAX = std::chrono::seconds(20);
 
 InfoBoxWindow::InfoBoxWindow(ContainerWindow &parent, PixelRect rc,
                              unsigned border_flags,
                              const InfoBoxSettings &_settings,
                              const InfoBoxLook &_look,
-                             const UnitsLook &_units_look,
                              unsigned _id,
                              WindowStyle style)
-  :content(NULL),
-   settings(_settings), look(_look), units_look(_units_look),
+  :settings(_settings), look(_look),
    border_kind(border_flags),
-   id(_id),
-   dragging(false), pressed(false),
-   force_draw_selector(false),
-   focus_timer(*this), dialog_timer(*this)
+   id(_id)
 {
   data.Clear();
 
-  style.EnableDoubleClicks();
   Create(parent, rc, style);
 }
 
-InfoBoxWindow::~InfoBoxWindow() {
-  delete content;
-  Destroy();
-}
-
 void
 InfoBoxWindow::SetTitle(const TCHAR *_title)
 {
@@ -93,32 +78,35 @@ InfoBoxWindow::PaintTitle(Canvas &canvas)
 
   PixelSize tsize = canvas.CalcTextSize(data.title);
 
-  PixelScalar halftextwidth = (title_rect.left + title_rect.right - tsize.cx) / 2;
-  PixelScalar x = std::max(PixelScalar(1),
-                           PixelScalar(title_rect.left + halftextwidth));
-  PixelScalar y = title_rect.top;
+  int halftextwidth = (title_rect.left + title_rect.right - (int)tsize.width) / 2;
+  int x = std::max(1, title_rect.left + halftextwidth);
+  int y = title_rect.top;
 
-  canvas.TextAutoClipped(x, y, data.title);
+  canvas.TextAutoClipped({x, y}, data.title);
 
   if (settings.border_style == InfoBoxSettings::BorderStyle::TAB &&
       halftextwidth > Layout::Scale(3)) {
-    PixelScalar ytop = title_rect.top + font.GetCapitalHeight() / 2;
-    PixelScalar ytopedge = ytop + Layout::GetTextPadding();
-    PixelScalar ybottom = title_rect.top + Layout::Scale(6)
+
+    const auto pad = Layout::Scale(2);
+    const auto text_pad = Layout::GetTextPadding()*2;
+
+    int ytop = title_rect.top + font.GetCapitalHeight() / 2;
+    int ytopedge = ytop + pad;
+    int ybottom = title_rect.top + Layout::Scale(6)
       + font.GetCapitalHeight();
 
     canvas.Select(look.border_pen);
 
-    RasterPoint tab[8];
+    BulkPixelPoint tab[8];
     tab[0].x = tab[1].x = title_rect.left;
     tab[0].y = tab[7].y = ybottom;
-    tab[2].x = title_rect.left + Layout::GetTextPadding();
+    tab[2].x = title_rect.left + pad;
     tab[2].y = tab[5].y = tab[3].y = tab[4].y = ytop;
     tab[1].y = tab[6].y = ytopedge;
-    tab[5].x = title_rect.right - Layout::GetTextPadding();
+    tab[5].x = title_rect.right - pad;
     tab[6].x = tab[7].x = title_rect.right;
-    tab[3].x = title_rect.left + halftextwidth - Layout::Scale(1);
-    tab[4].x = title_rect.right - halftextwidth + Layout::Scale(1);
+    tab[3].x = title_rect.left + halftextwidth - text_pad;
+    tab[4].x = title_rect.right - halftextwidth + text_pad;
 
     canvas.DrawPolyline(tab, 4);
     canvas.DrawPolyline(tab + 4, 4);
@@ -133,87 +121,34 @@ InfoBoxWindow::PaintValue(Canvas &canvas, Color background_color)
 
   canvas.SetTextColor(look.GetValueColor(data.value_color));
 
-#ifndef GNAV
-  // Do text-based unit rendering on higher resolutions
-  if (look.unit_font.IsDefined()) {
-    canvas.Select(look.unit_font);
-    PixelScalar unit_width =
-        UnitSymbolRenderer::GetSize(canvas, data.value_unit).cx;
-
-    canvas.Select(look.value_font);
-    int ascent_height = look.value_font.GetAscentHeight();
-
-    PixelSize value_size = canvas.CalcTextSize(data.value);
-    if (value_size.cx + unit_width > value_rect.right - value_rect.left) {
-      canvas.Select(look.small_value_font);
-      ascent_height = look.small_value_font.GetAscentHeight();
-      value_size = canvas.CalcTextSize(data.value);
-    }
-
-    PixelScalar x = std::max(PixelScalar(0),
-                             PixelScalar((value_rect.left + value_rect.right
-                                          - value_size.cx - unit_width) / 2));
-
-    PixelScalar y = (value_rect.top + value_rect.bottom - value_size.cy) / 2;
-
-    canvas.TextAutoClipped(x, y, data.value);
-
-    if (unit_width != 0) {
-      const int unit_height =
-        UnitSymbolRenderer::GetAscentHeight(look.unit_font, data.value_unit);
-
-      canvas.Select(look.unit_font);
-      UnitSymbolRenderer::Draw(canvas,
-                               { x + value_size.cx,
-                                 y + ascent_height - unit_height },
-                               data.value_unit, look.unit_fraction_pen);
-    }
-    return;
-  }
-#endif
-
   canvas.Select(look.value_font);
-  UPixelScalar ascent_height = look.value_font.GetAscentHeight();
-  UPixelScalar capital_height = look.value_font.GetCapitalHeight();
-
-  PixelSize unit_size;
-  const UnitSymbol *unit_symbol = units_look.GetSymbol(data.value_unit);
-  if (unit_symbol != NULL) {
-    unit_size = unit_symbol->GetSize();
-  } else {
-    unit_size.cx = 0;
-    unit_size.cy = 0;
-  }
+  int ascent_height = look.value_font.GetAscentHeight();
 
   PixelSize value_size = canvas.CalcTextSize(data.value);
-  if (value_size.cx + unit_size.cx > value_rect.right - value_rect.left) {
+  if (unsigned(value_size.width + unit_width) > value_rect.GetWidth()) {
     canvas.Select(look.small_value_font);
     ascent_height = look.small_value_font.GetAscentHeight();
-    capital_height = look.small_value_font.GetCapitalHeight();
     value_size = canvas.CalcTextSize(data.value);
   }
 
-  PixelScalar x = std::max(PixelScalar(1),
-                           PixelScalar((value_rect.left + value_rect.right
-                                        - value_size.cx
-                                        - Layout::FastScale(unit_size.cx)) / 2));
+  const PixelSize value_unit_size = value_size + PixelSize{unit_width, 0u};
 
-  PixelScalar y = value_rect.top + 1 - ascent_height +
-    (value_rect.bottom - value_rect.top + capital_height) / 2;
+  auto value_p = value_rect.CenteredTopLeft(value_unit_size);
+  if (value_p.x < 0)
+    value_p.x = 0;
 
-  canvas.TextAutoClipped(x, y, data.value);
+  canvas.TextAutoClipped(value_p, data.value);
 
-  if (unit_symbol != NULL) {
-#ifndef HAVE_CLIPPING
-    /* sort-of clipping */
-    if (x + value_size.cx >= (int)canvas.GetWidth())
-      return;
-#endif
+  if (unit_width != 0) {
+    const int unit_height =
+      UnitSymbolRenderer::GetAscentHeight(look.unit_font, data.value_unit);
 
-    unit_symbol->Draw(canvas, x + value_size.cx,
-                      y + ascent_height - unit_symbol->GetScreenSize().cy,
-                      background_color,
-                      look.GetValueColor(data.value_color));
+    const auto unit_p = value_p.At(value_size.width,
+                                   ascent_height - unit_height);
+
+    canvas.Select(look.unit_font);
+    UnitSymbolRenderer::Draw(canvas, unit_p,
+                             data.value_unit, look.unit_fraction_pen);
   }
 }
 
@@ -230,12 +165,11 @@ InfoBoxWindow::PaintComment(Canvas &canvas)
 
   PixelSize tsize = canvas.CalcTextSize(data.comment);
 
-  PixelScalar x = std::max(PixelScalar(1),
-                           PixelScalar((comment_rect.left + comment_rect.right
-                                        - tsize.cx) / 2));
-  PixelScalar y = comment_rect.top;
+  int x = std::max(1,
+                   (comment_rect.left + comment_rect.right - (int)tsize.width) / 2);
+  int y = comment_rect.top;
 
-  canvas.TextAutoClipped(x, y, data.comment);
+  canvas.TextAutoClipped({x, y}, data.comment);
 }
 
 void
@@ -251,7 +185,7 @@ InfoBoxWindow::Paint(Canvas &canvas)
   else
     canvas.Clear(background_color);
 
-  if (data.GetCustom() && content != NULL) {
+  if (data.GetCustom() && content) {
     /* if there's no comment, the content object may paint that area,
        too */
     const PixelRect &rc = data.comment.empty()
@@ -269,48 +203,31 @@ InfoBoxWindow::Paint(Canvas &canvas)
   if (border_kind != 0) {
     canvas.Select(look.border_pen);
 
-    const UPixelScalar width = canvas.GetWidth(),
+    const int width = canvas.GetWidth(),
       height = canvas.GetHeight();
 
     if (border_kind & BORDERTOP) {
-      canvas.DrawExactLine(0, 0, width - 1, 0);
+      canvas.DrawExactLine({0, 0}, {width - 1, 0});
     }
 
     if (border_kind & BORDERRIGHT) {
-      canvas.DrawExactLine(width - 1, 0, width - 1, height);
+      canvas.DrawExactLine({width - 1, 0}, {width - 1, height});
     }
 
     if (border_kind & BORDERBOTTOM) {
-      canvas.DrawExactLine(0, height - 1, width - 1, height - 1);
+      canvas.DrawExactLine({0, height - 1}, {width - 1, height - 1});
     }
 
     if (border_kind & BORDERLEFT) {
-      canvas.DrawExactLine(0, 0, 0, height - 1);
+      canvas.DrawExactLine({0, 0}, {0, height - 1});
     }
   }
 }
 
 void
-InfoBoxWindow::PaintInto(Canvas &dest, PixelScalar xoff, PixelScalar yoff,
-                         UPixelScalar width, UPixelScalar height)
+InfoBoxWindow::SetContentProvider(std::unique_ptr<InfoBoxContent> _content)
 {
-#ifdef ENABLE_OPENGL
-  SubCanvas canvas(dest, RasterPoint(xoff, yoff), PixelSize(width, height));
-  Paint(canvas);
-#else
-  const PixelSize size = GetSize();
-  BufferCanvas buffer(dest, size);
-
-  Paint(buffer);
-  dest.Stretch(xoff, yoff, width, height, buffer, 0, 0, size.cx, size.cy);
-#endif
-}
-
-void
-InfoBoxWindow::SetContentProvider(InfoBoxContent *_content)
-{
-  delete content;
-  content = _content;
+  content = std::move(_content);
 
   data.SetInvalid();
   Invalidate();
@@ -319,7 +236,7 @@ InfoBoxWindow::SetContentProvider(InfoBoxContent *_content)
 void
 InfoBoxWindow::UpdateContent()
 {
-  if (content == NULL)
+  if (!content)
     return;
 
   InfoBoxData old = data;
@@ -342,6 +259,9 @@ InfoBoxWindow::UpdateContent()
     if (!data.CompareComment(old))
       Invalidate(comment_rect);
 #endif
+
+    unit_width = UnitSymbolRenderer::GetSize(look.unit_font,
+                                             data.value_unit).width;
   }
 }
 
@@ -358,7 +278,7 @@ InfoBoxWindow::ShowDialog()
 bool
 InfoBoxWindow::HandleKey(InfoBoxContent::InfoBoxKeyCodes keycode)
 {
-  if (content != NULL && content->HandleKey(keycode)) {
+  if (content && content->HandleKey(keycode)) {
     UpdateContent();
     return true;
   }
@@ -368,7 +288,7 @@ InfoBoxWindow::HandleKey(InfoBoxContent::InfoBoxKeyCodes keycode)
 const InfoBoxPanel *
 InfoBoxWindow::GetDialogContent() const
 {
-  if (content != NULL)
+  if (content)
     return content->GetDialogContent();
 
   return NULL;
@@ -405,7 +325,8 @@ InfoBoxWindow::OnResize(PixelSize new_size)
   title_rect.bottom = rc.top + look.title_font.GetHeight();
 
   comment_rect = rc;
-  comment_rect.top = comment_rect.bottom - (look.title_font.GetHeight() + 2);
+  comment_rect.top = comment_rect.bottom - look.title_font.GetHeight();
+
   value_rect = rc;
   value_rect.top = title_rect.bottom;
   value_rect.bottom = comment_rect.top;
@@ -457,7 +378,7 @@ InfoBoxWindow::OnKeyDown(unsigned key_code)
 }
 
 bool
-InfoBoxWindow::OnMouseDown(PixelScalar x, PixelScalar y)
+InfoBoxWindow::OnMouseDown(PixelPoint p)
 {
   dialog_timer.Cancel();
 
@@ -469,14 +390,14 @@ InfoBoxWindow::OnMouseDown(PixelScalar x, PixelScalar y)
     Invalidate();
 
     /* start "long click" detection */
-    dialog_timer.Schedule(1000);
+    dialog_timer.Schedule(std::chrono::seconds(1));
   }
 
   return true;
 }
 
 bool
-InfoBoxWindow::OnMouseUp(PixelScalar x, PixelScalar y)
+InfoBoxWindow::OnMouseUp(PixelPoint p)
 {
   dialog_timer.Cancel();
 
@@ -494,7 +415,7 @@ InfoBoxWindow::OnMouseUp(PixelScalar x, PixelScalar y)
 
       if (GetDialogContent() != nullptr)
         /* delay the dialog, so double click detection works */
-        dialog_timer.Schedule(300);
+        dialog_timer.Schedule(std::chrono::milliseconds(300));
     }
 
     return true;
@@ -504,21 +425,18 @@ InfoBoxWindow::OnMouseUp(PixelScalar x, PixelScalar y)
 }
 
 bool
-InfoBoxWindow::OnMouseDouble(PixelScalar x, PixelScalar y)
+InfoBoxWindow::OnMouseDouble(PixelPoint p)
 {
   dialog_timer.Cancel();
-
-  if (!IsAltair())
-    InputEvents::ShowMenu();
-
+  InputEvents::ShowMenu();
   return true;
 }
 
 bool
-InfoBoxWindow::OnMouseMove(PixelScalar x, PixelScalar y, unsigned keys)
+InfoBoxWindow::OnMouseMove(PixelPoint p, unsigned keys)
 {
   if (dragging) {
-    SetPressed(IsInside(x, y));
+    SetPressed(IsInside(p));
     if (!pressed)
       dialog_timer.Cancel();
     return true;
@@ -556,7 +474,7 @@ InfoBoxWindow::OnSetFocus()
 
   // Start the focus-auto-return timer
   // to automatically return focus back to MapWindow if idle
-  focus_timer.Schedule(HasCursorKeys() ? FOCUS_TIMEOUT_MAX : 1100);
+  focus_timer.Schedule(HasCursorKeys() ? FOCUS_TIMEOUT_MAX : std::chrono::milliseconds(1100));
 
   // Redraw fast to paint the selector
   Invalidate();
@@ -575,21 +493,12 @@ InfoBoxWindow::OnKillFocus()
   Invalidate();
 }
 
-bool
-InfoBoxWindow::OnTimer(WindowTimer &timer)
+void
+InfoBoxWindow::OnDialogTimer() noexcept
 {
-  if (timer == focus_timer) {
-    focus_timer.Cancel();
-    FocusParent();
-    return true;
-  } else if (timer == dialog_timer) {
-    dragging = pressed = false;
-    Invalidate();
-    ReleaseCapture();
+  dragging = pressed = false;
+  Invalidate();
+  ReleaseCapture();
 
-    dialog_timer.Cancel();
-    ShowDialog();
-    return true;
-  } else
-    return PaintWindow::OnTimer(timer);
+  ShowDialog();
 }
