@@ -390,18 +390,22 @@ InputEvents::IsGesture(const TCHAR *data) noexcept
 bool
 InputEvents::processGesture(const TCHAR *data) noexcept
 {
+  // start with lua event if available!
+  if (Lua::FireGesture(data))
+    return true;
+
   // get current mode
   unsigned event_id = gesture_to_event(data);
-  if (event_id)
-  {
+  if (event_id) {
     InputEvents::processGo(event_id);
     return true;
   }
-  return Lua::FireGesture(data);
+
+  return false;
 }
 
 /*
-  InputEvent::processNmea(TCHAR* data)
+  InputEvent::processNmea(TCHAR *data)
   Take hard coded inputs from NMEA processor.
   Return = TRUE if we have a valid key match
 */
