@@ -32,6 +32,8 @@ Copyright_License {
 #include "io/UniqueFileDescriptor.hxx"
 #include "util/Exception.hxx"
 
+#include <cwchar>
+
 #include <stdio.h>
 #include <stdarg.h>
 #include <windef.h> // for MAX_PATH
@@ -98,7 +100,7 @@ LogFormat(const char *fmt, ...) noexcept
   va_list ap;
 
   va_start(ap, fmt);
-  vsprintf(buf, fmt, ap);
+  vsnprintf(buf, sizeof(buf) - 1, fmt, ap);
   va_end(ap);
 
   LogString(buf);
@@ -107,25 +109,25 @@ LogFormat(const char *fmt, ...) noexcept
 #ifdef _UNICODE
 
 static void
-LogString(const TCHAR *p) noexcept
+LogString(const wchar_t *p) noexcept
 {
   TextWriter writer(OpenLog());
   if (!writer.IsOpen())
     return;
 
-  TCHAR time_buffer[32];
+  wchar_t time_buffer[32];
   FormatISO8601(time_buffer, BrokenDateTime::NowUTC());
-  writer.FormatLine(_T("[%s] %s"), time_buffer, p);
+  writer.FormatLine(L"[%s] %s", time_buffer, p);
 }
 
 void
-LogFormat(const TCHAR *Str, ...) noexcept
+LogFormat(const wchar_t *Str, ...) noexcept
 {
-  TCHAR buf[MAX_PATH];
+  wchar_t buf[MAX_PATH];
   va_list ap;
 
   va_start(ap, Str);
-  _vstprintf(buf, Str, ap);
+  std::vswprintf(buf, std::size(buf), Str, ap);
   va_end(ap);
 
   LogString(buf);
