@@ -124,6 +124,13 @@ private:
 
   bool restore_page_pending = false;
 
+  /**
+   * Has "late" initialization been done already?  Those are things
+   * that must be run from inside the main event loop.  It will be
+   * checked and set by OnTimer().
+   */
+  bool late_initialised = false;
+
 public:
   virtual ~MainWindow();
 
@@ -342,6 +349,8 @@ private:
 
   void StopDragging();
 
+  void LateInitialise() noexcept;
+
   void RunTimer() noexcept;
 
   void OnGpsNotify() noexcept;
