@@ -1,7 +1,7 @@
 /* Copyright_License {
 
   XCSoar Glide Computer - http://www.xcsoar.org/
-  Copyright (C) 2000-2015 The XCSoar Project
+  Copyright (C) 2000-2021 The XCSoar Project
   A detailed list of copyright holders can be found in the file "AUTHORS".
 
   This program is free software; you can redistribute it and/or
@@ -24,20 +24,20 @@
 #include "LoadFile.hpp"
 #include "LocalPath.hpp"
 #include "Engine/Task/Ordered/OrderedTask.hpp"
+#include "system/Path.hpp"
 
-#include <windef.h> // for MAX_PATH
-
-OrderedTask *
+std::unique_ptr<OrderedTask>
 LoadDefaultTask(const TaskBehaviour &task_behaviour,
-                const Waypoints *waypoints)
+                const Waypoints *waypoints) noexcept
 {
-  TCHAR buffer[MAX_PATH];
-  const auto path = LocalPath(buffer, default_task_path);
-  OrderedTask *task = LoadTask(path, task_behaviour, waypoints);
-  if (!task) {
-    task = new OrderedTask(task_behaviour);
+  const auto path = LocalPath(default_task_path);
+
+  try {
+    return LoadTask(path, task_behaviour, waypoints);
+  } catch (...) {
+    auto task = std::make_unique<OrderedTask>(task_behaviour);
     assert(task);
     task->SetFactory(task_behaviour.task_type_default);
+    return task;
   }
-  return task;
 }
