@@ -1,7 +1,7 @@
 /* Copyright_License {
 
   XCSoar Glide Computer - http://www.xcsoar.org/
-  Copyright (C) 2000-2015 The XCSoar Project
+  Copyright (C) 2000-2021 The XCSoar Project
   A detailed list of copyright holders can be found in the file "AUTHORS".
 
   This program is free software; you can redistribute it and/or
@@ -24,18 +24,18 @@
 #define XCSOAR_TASK_FILE_IGC_HPP
 
 #include "TaskFile.hpp"
+#include "system/Path.hpp"
 
 class TaskFileIGC: public TaskFile
 {
 public:
-  TaskFileIGC(const TCHAR* _path):
-    TaskFile(_path) {}
+  using TaskFile::TaskFile;
 
-  virtual OrderedTask *GetTask(const TaskBehaviour &task_behaviour,
-                               const Waypoints *waypoints,
-                               unsigned index) const;
+  std::vector<tstring> GetList() const override;
 
-  unsigned Count();
+  std::unique_ptr<OrderedTask> GetTask(const TaskBehaviour &task_behaviour,
+                                       const Waypoints *waypoints,
+                                       unsigned index) const override;
 };
 
 #endif
