@@ -2,7 +2,7 @@
 Copyright_License {
 
   XCSoar Glide Computer - http://www.xcsoar.org/
-  Copyright (C) 2000-2015 The XCSoar Project
+  Copyright (C) 2000-2021 The XCSoar Project
   A detailed list of copyright holders can be found in the file "AUTHORS".
 
   This program is free software; you can redistribute it and/or
@@ -24,15 +24,15 @@ Copyright_License {
 #ifndef TOPOGRAPHY_FILE_RENDERER_HPP
 #define TOPOGRAPHY_FILE_RENDERER_HPP
 
-#include "Screen/Pen.hpp"
-#include "Screen/Icon.hpp"
-#include "Util/Serial.hpp"
+#include "ui/canvas/Pen.hpp"
+#include "ui/canvas/Icon.hpp"
+#include "util/Serial.hpp"
 #include "Geo/GeoBounds.hpp"
 
 #ifdef ENABLE_OPENGL
-#include "Screen/OpenGL/Surface.hpp"
+#include "ui/canvas/opengl/Surface.hpp"
 #else
-#include "Screen/Brush.hpp"
+#include "ui/canvas/Brush.hpp"
 #include "Topography/ShapeRenderer.hpp"
 #endif
 
@@ -40,7 +40,7 @@ Copyright_License {
 
 class TopographyFile;
 class Canvas;
-class GLFallbackArrayBuffer;
+class GLArrayBuffer;
 class WindowProjection;
 class LabelBlock;
 class XShape;
@@ -77,7 +77,7 @@ class TopographyFileRenderer final
   std::vector<const XShape *> visible_shapes, visible_labels;
 
 #ifdef ENABLE_OPENGL
-  GLFallbackArrayBuffer *array_buffer;
+  GLArrayBuffer *array_buffer;
   Serial array_buffer_serial;
 #endif
 
