@@ -2,7 +2,7 @@
 Copyright_License {
 
   XCSoar Glide Computer - http://www.xcsoar.org/
-  Copyright (C) 2000-2015 The XCSoar Project
+  Copyright (C) 2000-2021 The XCSoar Project
   A detailed list of copyright holders can be found in the file "AUTHORS".
 
   This program is free software; you can redistribute it and/or
@@ -24,16 +24,21 @@ Copyright_License {
 #ifndef XCSOAR_TRACKING_SKYLINES_CLIENT_HPP
 #define XCSOAR_TRACKING_SKYLINES_CLIENT_HPP
 
-#include "Handler.hpp"
-#include "Net/AllocatedSocketAddress.hxx"
-#include "Net/SocketDescriptor.hpp"
-#include "IO/Async/SocketEventHandler.hpp"
+#include "event/SocketEvent.hxx"
+#include "net/AllocatedSocketAddress.hxx"
+#include "net/UniqueSocketDescriptor.hxx"
+#include "event/net/cares/SimpleResolver.hxx"
+#include "thread/Mutex.hxx"
+#include "util/Cancellable.hxx"
+#include "util/Compiler.h"
 
-#include <stdint.h>
+#include <cstdint>
+#include <optional>
 
 struct NMEAInfo;
 struct GeoPoint;
-class IOThread;
+
+namespace Cares { class Channel; }
 
 namespace SkyLinesTracking {
 
@@ -41,43 +46,55 @@ struct TrafficResponsePacket;
 struct UserNameResponsePacket;
 struct WaveResponsePacket;
 struct ThermalResponsePacket;
+class Handler;
 
-class Client
-#ifdef HAVE_SKYLINES_TRACKING_HANDLER
-  : private SocketEventHandler
-#endif
-{
-#ifdef HAVE_SKYLINES_TRACKING_HANDLER
-  IOThread *io_thread;
-  Handler *handler;
-#endif
+class Client final : Cares::SimpleHandler {
+  Handler *const handler;
 
-  uint64_t key;
+  /**
+   * Protects #resolving, #resolver, #socket.
+   */
+  mutable Mutex mutex;
+
+  uint64_t key = 0;
+
+  std::optional<Cares::SimpleResolver> resolver;
 
   AllocatedSocketAddress address;
-  SocketDescriptor socket;
+  UniqueSocketDescriptor socket;
+  SocketEvent socket_event;
 
 public:
-  Client()
-    :
-#ifdef HAVE_SKYLINES_TRACKING_HANDLER
-    io_thread(nullptr), handler(nullptr),
-#endif
-    key(0),
-    socket(SocketDescriptor::Undefined()) {}
+  explicit Client(EventLoop &event_loop,
+                  Handler *_handler=nullptr)
+    :handler(_handler),
+     socket_event(event_loop, BIND_THIS_METHOD(OnSocketReady)) {}
   ~Client() { Close(); }
 
+  auto &GetEventLoop() const noexcept {
+    return socket_event.GetEventLoop();
+  }
+
   constexpr
   static unsigned GetDefaultPort() {
     return 5597;
   }
 
-#ifdef HAVE_SKYLINES_TRACKING_HANDLER
-  void SetIOThread(IOThread *io_thread);
-  void SetHandler(Handler *handler);
-#endif
+  /**
+   * Is SkyLines tracking enabled in configuration?
+   */
+  bool IsEnabled() const {
+    return key != 0;
+  }
 
   bool IsDefined() const {
+    const std::lock_guard<Mutex> lock(mutex);
+    return resolver || socket.IsDefined();
+  }
+
+  gcc_pure
+  bool IsConnected() const {
+    const std::lock_guard<Mutex> lock(mutex);
     return socket.IsDefined();
   }
 
@@ -89,27 +106,31 @@ public:
     key = _key;
   }
 
+  void Open(Cares::Channel &cares, const char *server);
   bool Open(SocketAddress _address);
   void Close();
 
   template<typename P>
   bool SendPacket(const P &packet) {
+    const std::lock_guard<Mutex> lock(mutex);
     return socket.Write(&packet, sizeof(packet), address) == sizeof(packet);
   }
 
-  bool SendFix(const NMEAInfo &basic);
-  bool SendPing(uint16_t id);
+  void SendFix(const NMEAInfo &basic);
+  void SendPing(uint16_t id);
 
-  bool SendThermal(uint32_t time,
+  void SendThermal(uint32_t time,
                    ::GeoPoint bottom_location, int bottom_altitude,
                    ::GeoPoint top_location, int top_altitude,
                    double lift);
+  void SendThermalRequest();
 
-#ifdef HAVE_SKYLINES_TRACKING_HANDLER
-  bool SendTrafficRequest(bool followees, bool club, bool near);
-  bool SendUserNameRequest(uint32_t user_id);
+  void SendTrafficRequest(bool followees, bool club, bool near_);
+  void SendUserNameRequest(uint32_t user_id);
 
 private:
+  void InternalClose() noexcept;
+
   void OnTrafficReceived(const TrafficResponsePacket &packet, size_t length);
   void OnUserNameReceived(const UserNameResponsePacket &packet,
                           size_t length);
@@ -117,9 +138,11 @@ private:
   void OnThermalReceived(const ThermalResponsePacket &packet, size_t length);
   void OnDatagramReceived(void *data, size_t length);
 
-  /* virtual methods from SocketEventHandler */
-  bool OnSocketEvent(SocketDescriptor s, unsigned mask) override;
-#endif
+  void OnSocketReady(unsigned events) noexcept;
+
+  /* virtual methods from Cares::SimpleHandler */
+  void OnResolverSuccess(std::forward_list<AllocatedSocketAddress> addresses) noexcept override;
+  void OnResolverError(std::exception_ptr error) noexcept override;
 };
 
 } /* namespace SkyLinesTracking */
