@@ -23,12 +23,8 @@ Copyright_License {
 #if !defined(XCSOAR_UTILS_SYSTEM_H)
 #define XCSOAR_UTILS_SYSTEM_H
 
-#include <tchar.h>
-
 struct PixelSize;
 
-unsigned long FindFreeSpace(const TCHAR *path);
-
 PixelSize
 SystemWindowSize();
 
