@@ -36,7 +36,7 @@ Bitmap::Bitmap(ResourceId id)
   Load(id);
 }
 
-Bitmap::Bitmap(Bitmap &&src)
+Bitmap::Bitmap(Bitmap &&src) noexcept
   :bmp(src.bmp),
    uncompressed(std::move(src.uncompressed)),
    type(src.type),
@@ -55,7 +55,7 @@ Bitmap::Bitmap(Bitmap &&src)
 }
 
 static const char *
-find_resource_name(unsigned id)
+find_resource_name(unsigned id) noexcept
 {
   for (unsigned i = 0; DrawableNames[i].name != nullptr; ++i)
     if (DrawableNames[i].id == id)
@@ -64,7 +64,7 @@ find_resource_name(unsigned id)
   return nullptr;
 }
 
-static jobject
+static Java::LocalObject
 LoadResourceBitmap(ResourceId id)
 {
   const char *name = find_resource_name((unsigned)id);
@@ -75,13 +75,12 @@ LoadResourceBitmap(ResourceId id)
 }
 
 bool
-Bitmap::Set(JNIEnv *env, jobject _bmp, Type _type, bool flipped)
+Bitmap::Set(JNIEnv *env, jobject _bmp, Type _type, bool flipped) noexcept
 {
   assert(bmp == nullptr);
   assert(_bmp != nullptr);
 
   bmp = env->NewGlobalRef(_bmp);
-  env->DeleteLocalRef(_bmp);
 
   type = _type;
 
@@ -99,7 +98,7 @@ Bitmap::Set(JNIEnv *env, jobject _bmp, Type _type, bool flipped)
 }
 
 bool
-Bitmap::MakeTexture(jobject _bmp, Type _type, bool flipped)
+Bitmap::MakeTexture(jobject _bmp, Type _type, bool flipped) noexcept
 {
   assert(_bmp != nullptr);
 
@@ -124,11 +123,11 @@ Bitmap::Load(ResourceId id, Type _type)
 
   Reset();
 
-  auto *new_bmp = LoadResourceBitmap(id);
+  auto new_bmp = LoadResourceBitmap(id);
   if (new_bmp == nullptr)
     return false;
 
-  return Set(Java::GetEnv(), new_bmp, _type);
+  return Set(new_bmp.GetEnv(), new_bmp, _type);
 }
 
 bool
@@ -138,7 +137,7 @@ Bitmap::LoadFile(Path path)
 
   Reset();
 
-  jobject new_bmp;
+  Java::LocalObject new_bmp;
   bool flipped = false;
   if (path.MatchesExtension(_T(".tif")) || path.MatchesExtension(_T(".tiff"))) {
     new_bmp = native_view->loadFileTiff(path);
@@ -153,7 +152,7 @@ Bitmap::LoadFile(Path path)
 }
 
 void
-Bitmap::Reset()
+Bitmap::Reset() noexcept
 {
   if (bmp != nullptr) {
     auto *env = Java::GetEnv();
