@@ -28,6 +28,12 @@ Copyright_License {
 
 #include <cassert>
 
+Bitmap::Bitmap(Bitmap &&src) noexcept
+  :buffer(src.buffer)
+{
+  src.buffer = {};
+}
+
 bool
 Bitmap::Load(UncompressedImage &&uncompressed, gcc_unused Type type)
 {
@@ -41,7 +47,7 @@ Bitmap::Load(UncompressedImage &&uncompressed, gcc_unused Type type)
 }
 
 void
-Bitmap::Reset()
+Bitmap::Reset() noexcept
 {
   assert(!IsDefined() || IsScreenInitialized());
 
