@@ -2,7 +2,7 @@
 Copyright_License {
 
   XCSoar Glide Computer - http://www.xcsoar.org/
-  Copyright (C) 2000-2015 The XCSoar Project
+  Copyright (C) 2000-2021 The XCSoar Project
   A detailed list of copyright holders can be found in the file "AUTHORS".
 
   This program is free software; you can redistribute it and/or
@@ -24,8 +24,8 @@ Copyright_License {
 #include "Geo/Geoid.hpp"
 #include "Geo/GeoPoint.hpp"
 
-fixed
-EGM96::LookupSeparation(gcc_unused const GeoPoint &pt)
+double
+EGM96::LookupSeparation(const GeoPoint &)
 {
-  return fixed(0);
+  return 0;
 }
